<?php

// vim: fdm=marker

/**********************
*  Andy Geiss 2009    *
*  Tabulatorweite: 4  *
**********************/

require_once('../functions.inc.php');

class scalarvis_admin{
    /* {{{ Member-Variablen */
    public  $USER_CONFIG = array();         // Konfiguration
    private $session_name;                  // Session-Name
    private $session_id;                    // Session-ID
    private $user;                          // Scalarvis-User
    private $port;                          // Scalarvis-Port
    private $host;                          // Scalarvis-Host
    private $pwd;                           // Scalarvis-Passwort
    public  $lang_id;                       // Sprachschlssel (de, en, etc.)
    private $socket;                        // TCP/IP-Socketverbindung
    private $socket_errno;                  // Socket-Fehlernummer
    private $socket_last_error;             // Letzter Socket-Fehler aus socket_last_error()
    public  $sysinfo = array();             // Scalarvis-Sysinfo-Array
    private $MESSAGE_SIZE = 8192;           // Maximale Paketgre beim Blockdatentransfer
    /* }}} */

    /* {{{ Konstruktor */
    function __construct(){
        // Session-Info belegen
        $this->session_name = session_name();
        $this->session_id = session_id();

        // Auf erforderliche Extensions prfen
        $extension_array = array();
        $i = 0;
        $extension_array[$i]['name'] = 'sockets';
        $extension_array[$i]['file'] = 'php_sockets.dll';

        // L TODO: OS bercksichtigen
        foreach($extension_array as $extension){
            if(!extension_loaded($extension['name'])){
                if(!dl($extension['file'])){
                    echo '<br />Could not load ',$extension['file'],' dynamicly! <br />Please check your \'extension_dir\' setting in php.ini!';
                }
            }
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion config_checkbox() - Config-Checkbox generieren */
    public function config_checkbox($field_name){
        $retval = '<input type="checkbox" name="'.$field_name.'"';
        if($this->USER_CONFIG[$field_name]){
            $retval .= ' checked';
        }
        $retval .= ' />';

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion config_input_text() - Config-Option fr Select-Container generieren */
    public function config_input_text($field_name){
        $retval = '<input type="text" name="'.$field_name.'"';
        $retval .= ' value="'.$this->USER_CONFIG[$field_name].'"';
        $retval .= ' class="admin" />';

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion config_select_option() - Config-Option fr Select-Container generieren */
    public function config_select_option($field_name, $value, $innerHTML = ''){
        if(empty($innerHTML)){
            $innerHTML = $value;
        }

        $retval = '<option value="'.$value.'"';
        if(isset($this->USER_CONFIG[$field_name])
        && (empty($this->USER_CONFIG[$field_name]) && empty($value)
        || strtolower(trim($this->USER_CONFIG[$field_name])) == strtolower(trim($value)))){
            $retval .= ' selected';
        }
        $retval .= '>'.$innerHTML.'</option>';

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion connect() - Verbindung zu Scalarvis herstellen */
    public function connect($user = null, $host = null, $port = null, $pwd = null, $cpwd = null){
        if(empty($user) && isset($_POST['user'])) $user = $_POST['user'];
        if(empty($host) && isset($_POST['host'])) $host = $_POST['host'];
        if(empty($port) && isset($_POST['port'])) $port = $_POST['port'];
        if(empty($pwd) && isset($_POST['pwd'])) $pwd = $_POST['pwd'];
        if(empty($cpwd) && isset($_POST['cpwd'])) $cpwd = $_POST['cpwd'];
        $this->socket_errno = 0;
        $this->socket_last_error = 0;

        if(empty($this->user) && empty($this->port)){
            if(empty($user)){
                $this->socket_errno = 1;
                return false;
            }
            else{
                $this->user = $user;
            }

            if(empty($host)){
                $this->socket_errno = 2;
                return false;
            }
            else{
                $this->host = $host;
            }

            if(empty($port)){
                $this->socket_errno = 3;
                return false;
            }
            else{
                $this->port = $port;
            }

            if(empty($pwd)){
                if(!empty($cpwd)){
                    $decoded_array = time_decrypt($cpwd);
                    if(isset($STANDARD_CONFIG['VALID_DURATION']) && is_numeric($STANDARD_CONFIG['VALID_DURATION'])){
                        $valid_duration = $STANDARD_CONFIG['VALID_DURATION'];
                    }
                    else{
                        $valid_duration = 600;  // standardmig 10 Minuten
                    }
                    // Wenn Passwort noch gltig
                    if($decoded_array['decoded_timestamp'] + $valid_duration > time()){
                        $this->pwd = $decoded_array['decoded_value'];
                    }
                }
                else{
                    $this->socket_errno = 4;
                    return false;
                }
            }
            else{
                $this->pwd = $pwd;
            }
        }

        $this->socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if(!$this->socket){
            $this->socket_errno = 5;
            $this->socket_last_error = socket_last_error();
            return false;
        }

        $prev_error_reporting = error_reporting(0);
        if(!socket_connect($this->socket, $this->host, $this->port)){
            $this->socket_errno = 6;
            $this->socket_last_error = socket_last_error($this->socket);
            socket_close($this->socket);
            return false;
        }
        error_reporting($prev_error_reporting);

        $socket_buffer = 'SYSTAGNR,'.$this->user.','.psgencrypt($this->pwd);
        socket_write_all($this->socket, $socket_buffer);
        $socket_buffer = socket_read_all($this->socket, 512);

        // Prfen, ob die Serververbindung steht
        if(!strlen($socket_buffer) || stristr($socket_buffer, 'ERROR')){
            $this->socket_errno = 7;
            return false;
        }
        else{
            list($this->sysinfo['tag_anzahl'],
                 $this->sysinfo['intervall'],
                 $this->sysinfo['refresh'],
                 $this->sysinfo['bezeichnung'],
                 $this->sysinfo['kurven_anzahl'],
                 $this->sysinfo['stoer_intervall'],
                 $this->sysinfo['rohkanal_anzahl'],
                 $this->sysinfo['version']) = explode(',',$socket_buffer);
            $this->sysinfo['bezeichnung'] = trim($this->sysinfo['bezeichnung']);
        }

        return true;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_host() - Getter fr host */
    public function get_host(){
        return $this->host;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_port() - Getter fr port */
    public function get_port(){
        return $this->port;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_session_name() - Getter fr session_name */
    public function get_session_name(){
        return $this->session_name;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_socket_errno() - Getter fr socket_errno */
    public function get_socket_errno(){
        return $this->socket_errno;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_socket_last_error() - Getter fr socket_last_error */
    public function get_socket_last_error(){
        return $this->socket_last_error;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_user() - Getter fr user */
    public function get_user(){
        return $this->user;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_user_property() - Liefert zum Namen das User-Property vom Scalarvis-Server */
    private function get_user_property($name){
        $socket_buffer = sprintf('GETUSERPROPERTY,%s', $name);
        socket_write_all($this->socket, $socket_buffer);
        $socket_buffer = socket_read_all($this->socket, $this->MESSAGE_SIZE);
        if(stristr($socket_buffer, 'ERROR')){
            return false;
        }
        else{
            $socket_buffer = substr($socket_buffer, 16); // Getuserproperty abschneiden
            $socket_buffer = substr($socket_buffer, (strpos($socket_buffer, ',') + 1)); // Namen und Komma abschneiden
            if(trim($socket_buffer) == '!'){
                return false;
            }
        }
        return $socket_buffer;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion init_config_field() - Config-Feld initialisieren */
    public function init_config_field($field_name, $init_value = 0){
        global $STANDARD_CONFIG;

        if(isset($_POST[$field_name])){
            $init_value = $_POST[$field_name];
        }
        else if(isset($this->USER_CONFIG[$field_name])){
            // Beim Submit nicht vorhandenes Boolean-Feld abhaken
            if(!empty($_POST['submit_action'])){
                $init_value = 0;
            }
            else{
                $init_value = $this->USER_CONFIG[$field_name];
            }
        }
        else if(isset($STANDARD_CONFIG[$field_name])){
            $init_value = $STANDARD_CONFIG[$field_name];
        }

        $this->USER_CONFIG[$field_name] = $init_value;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion load_user_properties() - Ldt die User-Konfiguration */
    public function load_user_properties(){
        global $STANDARD_CONFIG;
        global $g_path_separator;
        global $g_user_config_field_array;
        $retval = false;

        foreach($g_user_config_field_array as $config_field){
            if(isset($STANDARD_CONFIG[$config_field])){
                $this->USER_CONFIG[$config_field] = $STANDARD_CONFIG[$config_field];
            }
        }

        if(!empty($STANDARD_CONFIG['USER_CONFIG_FILES'])){
            $conf_dir = '..'.$g_path_separator.'user_config';
            $conf_filename = str_replace('.', '_', strtolower($this->user.'__'.$this->host.'__'.$this->port)).'.conf';

            if(is_file($conf_dir.$g_path_separator.$conf_filename)){
                if($conf_fp = fopen($conf_dir.$g_path_separator.$conf_filename, 'r')){
                    $socket_buffer = fgets($conf_fp, $this->MESSAGE_SIZE);
                    fclose($conf_fp);
                    $retval = true;
                }
            }
        }
        else{
            $socket_buffer = $this->get_user_property('WEB$CONF');
        }

        // Wenn es Daten gibt
        if(!empty($socket_buffer) && strlen(trim($socket_buffer))){
            $socket_buffer = __unserialize($socket_buffer);
            if(is_array($socket_buffer)){
                $keys = array_keys($socket_buffer);
                for($i = 0; $i < count($keys); $i++){
                    $this->USER_CONFIG[$keys[$i]] = $socket_buffer[$keys[$i]];
                }
                $retval = true;
            }
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion save_user_properties() - Speichert die User-Konfiguration */
    public function save_user_properties(){
        global $STANDARD_CONFIG;
        global $g_path_separator;
        $retval = false;

        if(!empty($STANDARD_CONFIG['USER_CONFIG_FILES'])){
            $conf_dir = '..'.$g_path_separator.'user_config';
            $conf_filename = str_replace('.', '_', strtolower($this->user.'__'.$this->host.'__'.$this->port)).'.conf';

            // Wenn es das user_config-Verzeichnis nicht gibt, anlegen
            if(!is_dir($conf_dir)){
                mkdir($conf_dir);
            }

            if($conf_fp = fopen($conf_dir.$g_path_separator.$conf_filename, 'w')){
                $socket_buffer = serialize($this->USER_CONFIG);
                fwrite($conf_fp, $socket_buffer);
                fclose($conf_fp);
                $retval = true;
            }
        }
        else{
            $socket_buffer = sprintf('SETUSERPROPERTY,%s,%s', 'WEB$CONF', serialize($this->USER_CONFIG));
            // N TODO: Profil wird beim Speichern ber den Socket willkrlich zerstrt/abgeschnitten
            // L TODO: Base64 verwenden zum Serialisieren?
            //$socket_buffer = sprintf('SETUSERPROPERTY,%s,%s', 'WEB$CONF', base64_encode(serialize($this->USER_CONFIG)));
            socket_write_all($this->socket, $socket_buffer);
            $socket_buffer = socket_read_all($this->socket, $this->MESSAGE_SIZE);
            if(!stristr($socket_buffer, 'ERROR')){
                $retval = true;
            }
        }

        return $retval;
    }
    /* }}} */
}
?>
