<?php

// vim: fdm=marker

/* * *************************
 *  Andy Geiss 2006 - 2010  *
 *  Tabulatorweite: 4       *
 * ************************* */

// Fliebild-Funktionen


/* {{{ PHP-Funktion get_fliess_array() - Liest Fliess-Section des INI-Arrays in ein assoziatives Array ein und gibt dieses zurck */
function get_fliess_array($ini_array) {
    $fliess_array = array();

    foreach ($ini_array as $section_index => $section_array) {
        if (substr($section_index, 0, 6) == 'Fliess') {
            // Section-Werte ermitteln
            SetSectionValueToFliessArray('Bild', $section_array, $fliess_array);
            SetSectionValueToFliessArray('FliessCaption', $section_array, $fliess_array);
            SetSectionValueToFliessArray('ErstesRegister', $section_array, $fliess_array);
            SetSectionValueToFliessArray('PosHeight', $section_array, $fliess_array);
            SetSectionValueToFliessArray('PosLeft', $section_array, $fliess_array);
            SetSectionValueToFliessArray('PosTop', $section_array, $fliess_array);
            SetSectionValueToFliessArray('PosWidth', $section_array, $fliess_array);
            SetSectionValueToFliessArray('ScalMinSize', $section_array, $fliess_array);
            SetSectionValueToFliessArray('TabPerRow', $section_array, $fliess_array);
            SetSectionValueToFliessArray('ServerTime', $section_array, $fliess_array);
            SetSectionValueToFliessArray('StartModus', $section_array, $fliess_array);
        }
    }

    return $fliess_array;
}

/* }}} */

/* {{{ PHP-Funktion get_ini_array() - Liest ein INI-File in ein strukturiertes assoziatives Array ein und gibt dieses zurck */

function get_ini_array($ini_filename, $buffer_len = 1024) {
    $ini_array = array();
    $current_section = '';
    $fp = fopen($ini_filename, "r");
    if ($fp) {
        while (!feof($fp)) {
            $buffer = trim(fgets($fp, $buffer_len));
            $buffer_strlen = strlen($buffer);
            // Wenn die Zeile einen Section-Beginn darstellt
            if ($buffer_strlen && substr($buffer, 0, 1) == '[' && substr($buffer, $buffer_strlen - 1, 1) == ']') {
                $current_section = substr($buffer, 1, $buffer_strlen - 2);
            } else if ($buffer_strlen && strlen($current_section)) {
                $buffer_explode = explode('=', $buffer, 2);
                $ini_array_index = trim($buffer_explode[0]);
                if (isset($buffer_explode[1])) {
                    $ini_array_value = trim($buffer_explode[1]);
                } else {
                    $ini_array_value = '';
                }
                $ini_array[$current_section][$ini_array_index] = $ini_array_value;
            }
        }
        fclose($fp);
    }

    return $ini_array;
}

/* }}} */

/**
 * Hilfsfunktion: Schreibt das bergebene Array in die angegebene Datei
 * ACHTUNG: Dabei werden ALLE Daten neu geschrieben
 *
 * @param string $filename
 * @param array $ini_array
 */
function write_ini_array_to_file($filename, $ini_array) {
    $output = '';
    foreach ($ini_array as $section => $values) {
        //values must be an array
        if (!is_array($values)) {
            continue;
        }
        //add section
        $output .= "[$section]" . PHP_EOL;
        //add key/value pairs
        foreach ($values as $key => $val) {
            $output .= "$key=$val" . PHP_EOL;
        }
        $output .= PHP_EOL;
    }
    //write data to file
    file_put_contents($filename, trim($output));
}

/* {{{ PHP-Funktion get_object_array() - Objekt-Array zum Register ermitteln */

function get_object_array($ini_array, $register) {
    $object_array = array();

    // Attribut-Relevanz
    foreach ($ini_array as $section_index => $section_array) {
        if (substr($section_index, 0, 10) == 'Objekt' . $register . '.') {
            $object_no = substr($section_index, 10);
            $object_array[$object_no]['ObjectNo'] = $object_no;
            /*
              echo '<pre>';
              echo "\r\nx_len: ",$section_index;
              echo "\r\ny_len: ",$object_no;
              echo "\r\ny_len: ",$section_array['TagNummer'];
              echo "\r\ny_len: ",$object_array[$object_no]['ObjectNo'];
              echo '</pre>';
             */
            // Anzahl-Tags-Relevanz
            // Section-Werte ermitteln
            SetSectionValueToObjectArray($object_no, 'Caption', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Maske', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Max', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Min', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Picture', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'PosHeight', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'PosLeft', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'PosTop', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'PosWidth', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'InnerBottom', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'InnerLeft', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'InnerRight', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'InnerTop', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Style', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagTyp', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TxtErr', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TxtFalse', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TxtTrue', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TxtTrue1', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TxtTrue2', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TxtTrue3', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'ValTrue1', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'ValTrue2', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'ValTrue3', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Font', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Ebene', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNummer', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'ForeColor', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'BackColor', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr2', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor2', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth2', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale2', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr3', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor3', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth3', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale3', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr4', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor4', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth4', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale4', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr5', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor5', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth5', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale5', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr6', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor6', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth6', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale6', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr7', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor7', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth7', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale7', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr8', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor8', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth8', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale8', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr9', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor9', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth9', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale9', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr10', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor10', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth10', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale10', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr11', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor11', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth11', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale11', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr12', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor12', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth12', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale12', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr13', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor13', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth13', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale13', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr14', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor14', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth14', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale14', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr15', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor15', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth15', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale15', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagNr16', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineColor16', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'LineWidth16', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'yScale16', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TimeSpan', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Archivart', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'AutoScaling', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Refresh', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'Alignment', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'NeedleWidth', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'TagEinheit', $section_array, $object_array);
            SetSectionValueToObjectArray($object_no, 'HyperLink', $section_array, $object_array);
        }
    }

    return $object_array;
}

/* }}} */

/* {{{ PHP-Funktion get_value_from_ini_array() - Ermittelt einen bestimmten Wert einer Section eines INI-Arrays */

function get_value_from_ini_array($ini_array, $section, $valindex) {
    $value = '';
    if (isset($ini_array[$section][$valindex])) {
        $value = $ini_array[$section][$valindex];
    }
    return $value;
}

/* }}} */

/* {{{ PHP-Funktion get_value_from_object_array() - Ermittelt einen bestimmten Wert einer Section eines Object-Arrays */

function get_value_from_object_array($object_array, $index, $valindex) {
    $value = '';
    if (isset($object_array[$index][$valindex])) {
        $value = $object_array[$index][$valindex];
    }
    return $value;
}

/* }}} */

/* {{{ PHP-Funktion GetGIFFile() - Wenn Bildobjekt GIF, Bilddateinamen zurckgeben */

function GetGIFFile($picture_object, $flowchart_dir) {
    $picture_file = '';
    $file_type = 0;

    $picture = StripPathAndExtension($picture_object, true);

    // X TODO: Prfen, ob File existiert
    if (is_file($flowchart_dir . $picture)) {
        $info = GetImageSize($flowchart_dir . $picture);
        $file_type = $info[2];
    }

    // Wenn ja (1 == GIF), Filenamen zurckgeben
    if ($file_type == 1) {
        $picture_file = $picture;
    }

    return $picture_file;
}

/* }}} */

/* {{{ PHP-Funktion GetPictureFileType() - Bilddateityp ermitteln */

function GetPictureFileType($flowchart_dir, $picture) {
    // Picture File Type ermitteln
	

	
    if (is_file($flowchart_dir . $picture . '.png')) {
        $info = GetImageSize($flowchart_dir . $picture . '.png');
        $file_type = $info[2];
    } else if (is_file($flowchart_dir . $picture . '.jpg')) {
        $info = GetImageSize($flowchart_dir . $picture . '.jpg');
        $file_type = $info[2];
    } else if (is_file($flowchart_dir . $picture . '.jpeg')) {
        $info = GetImageSize($flowchart_dir . $picture . '.jpeg');
        $file_type = $info[2];
	} else if (is_file($flowchart_dir . $picture . '.svg')) {
        //$info = GetImageSize($flowchart_dir . $picture . '.svg');
        //$file_type = $info[2];
		$file_type = 87; //this my numbber for picture file of type SVG
    } else {
        $file_type = 0;
    }
    return $file_type;
}

/* }}} */

/* {{{ PHP-Funktion SetSectionValueToFliessArray() - Hilfsfunktion fr get_fliess_array() */

function SetSectionValueToFliessArray($valindex, $section_array, &$fliess_array) {
    if (isset($section_array[$valindex])) {
        $fliess_array[$valindex] = $section_array[$valindex];
    }
}

/* }}} */

/* {{{ PHP-Funktion SetSectionValueToObjectArray() - Hilfsfunktion fr get_object_array() */

function SetSectionValueToObjectArray($object_no, $valindex, $section_array, &$object_array) {
    if (isset($section_array[$valindex])) {
        $object_array[$object_no][$valindex] = $section_array[$valindex];
    }
}

/* }}} */

/* {{{ PHP-Funktion StripPathAndExtension() - Pfad und Dateiextension entfernen */

function StripPathAndExtension($file, $leave_extension = false) {
    global $g_path_separator;

    // Pfadangabe entfernen
    $pos = strrpos($file, $g_path_separator);
    if ($pos) {
        $file = substr($file, $pos + 1);
    }

    if (!$leave_extension) {
        // Extension entfernen
        $pos = strrpos($file, '.');
        if ($pos) {
            $file = substr($file, 0, $pos);
        }
    }

    return $file;
}

/* }}} */
?>

