<?php

// vim: fdm=marker

/***************************
*  Andy Geiss 2009 - 2010  *
*  Tabulatorweite: 4       *
***************************/

// N TODO: Auch in Fliebild-Editor-Vorschau socket_read_all()/socket_write_all()?
// N TODO: Neue PHP-Version verwenden, wenn alte Scalarvis-Extension nicht mehr bentigt wird
// L TODO: $STANDARD_CONFIG['USE_PECL'] in FE-Vorschau bercksichtigen (durchschleifen?)

// L TODO: Draw-Objekte inkl. smtlicher Eigenschaften als XML an JavaScript bergeben? -> SVG?
// L TODO: yBase 0,1,2 bercksichtigen?

/* {{{ PHP-Klasse scalarvis_ext - Scalarvis-Extension als PHP-Klasse (Hauptklasse) */
class scalarvis_ext{
    /* {{{ Member-Variablen */
    private $socket;                        // TCP/IP-Socketverbindung
    private $sysinfo = array();             // Scalarvis-Sysinfo-Array
    private $MAX_AW_TAGS = 64;              // Maximale Anzahl an Tags, die gleichzeitig beim Blockdatentransfer angefragt werden knnen
    private $MESSAGE_SIZE = 8192;           // Maximale Paketgre beim Blockdatentransfer
    private $svsc = null;                   // Scalarvis-Short-Class-Instanz
    private $time_start = null;             // Start-Zeit
    private $time_end = null;               // Ende-Zeit
    private $type = '';                     // Diagramm-Typ
    private $returnImage = false;           // Wenn true, wird die Image-Ressource zurckgegeben statt ausgegeben
    private $is_shrinked = false;           // Kennzeichnet, ob Image verkleinert wurde
    private $is_ribbonplot = false;         // Kennzeichnet, ob Ribbonplot berechnet werden soll
    private $CodeBase = '';                 // CodeBase, z.B. '../' (fr Fonts)
    private $data = null;                   // Darzustellende Daten inkl. Block-Info
    private $data_ext = array();            // Zustzlich darzustellende Daten fr Vergleichsdiagramm
    private $tagauswahl = array();          // Ausschlaggebendes Tag-Struktur-Array nach Tag-Index

    private $AutoScale = array();           // AutoScale-Array-Struktur
    private $ImageResource = null;          // Image-Ressource
    private $BackgroundColor = null;        // Hintergrundfarbe
    private $Color = null;                  // Vordergrundfarbe
    private $PlotArea = null;               // Instanz des Zeichenbereichs zur Darstellung des Graphen
    private $Plot = array();                // Plot(s)
    private $VertAxis = array();            // Vertikale Achse(n)
    private $HorAxis = array();             // Horizontale Achse(n)

    private $ImageWidth = 800;              // Image-Gesamtbreite
    private $ImageHeight = 500;             // Image-Gesamthhe
    private $LeftMargin = 20;               // Grafikrand links
    private $RightMargin = 20;              // Grafikrand rechts
    private $TopMargin = 30;                // Grafikrand oben
    private $BottomMargin = 70;             // Grafikrand unten
    private $AxisWidth = 40;                // Achsenbreite bezglich Einteilung
    private $Axes = 0;                      // Anzahl Achsen resp. darzustellende Tags
		//private $tempShip = array();
		//private $testArray = null; //array();
    /* }}} */

    /* {{{ PHP-Klassenfunktion array_to_string() - Hilfsfunktion fr Getblockinfo */
    private function array_to_string($in){
        while(list($k, $v) = each($in)){
            if(isset($out)){
                $out = $out.','.$v;
            }
            else{
                $out = $v;
            }
        }
        return $out;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion connect() - Verbindung zu Scalarvis herstellen */
    private function connect(){
        $this->socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if(!$this->socket){
            return false;
        }

        $prev_error_reporting = error_reporting(0);
        if(!socket_connect($this->socket, $this->svsc->host, $this->svsc->port)){
            socket_close($this->socket);
            return false;
        }
        error_reporting($prev_error_reporting);

        $socket_buffer = 'SYSTAGNR,'.$this->svsc->user.','.psgencrypt($this->svsc->pwd);
        socket_write_all($this->socket, $socket_buffer);
        $socket_buffer = socket_read_all($this->socket, 512);

        // Prfen, ob die Serververbindung steht
        if(!strlen($socket_buffer) || stristr($socket_buffer, 'ERROR')){
            return false;
        }
        else{
            list($this->sysinfo['tag_anzahl'],
                 $this->sysinfo['intervall'],
                 $this->sysinfo['refresh'],
                 $this->sysinfo['bezeichnung'],
                 $this->sysinfo['kurven_anzahl'],
                 $this->sysinfo['stoer_intervall'],
                 $this->sysinfo['rohkanal_anzahl'],
                 $this->sysinfo['version']) = explode(',',$socket_buffer);
            $this->sysinfo['bezeichnung'] = trim($this->sysinfo['bezeichnung']);
        }

        return true;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Draw() - Zeichnet das Diagramm */
    private function Draw(){
        // N TODO: Keine-Daten-Meldung integrieren
        $this->SetAutoScale();
        if($this->is_ribbonplot){
            if(!$this->ImageResource = @ImageCreateTrueColor($this->ImageWidth, $this->ImageHeight)){
                return false;
            }
        }
        else{
            if(!$this->ImageResource = @ImageCreate($this->ImageWidth, $this->ImageHeight)){
                return false;
            }
        }

        $this->BackgroundColor = ImageColorAllocate($this->ImageResource, 200, 200, 200);
        $this->Color = ImageColorAllocate($this->ImageResource, 0, 0, 128);

        if($this->is_ribbonplot){
            ImageFill($this->ImageResource, 0, 0, $this->BackgroundColor);
        }

        $AxesLeft = (int) floor(($this->Axes + 1) / 2);
        $AxesRight = $this->Axes - $AxesLeft;
        $this->SetGraphMargins($this->LeftMargin + $AxesLeft * $this->AxisWidth,
                               $this->RightMargin + $AxesRight * $this->AxisWidth,
                               $this->TopMargin,
                               $this->BottomMargin);

        // Plot Area
        $this->PlotArea = new scalarvis_ext_plot_area;
        $this->PlotArea->SetCodeBase($this->CodeBase);
        // N TODO: Keine gelbe Standard-Linienfarbe -> auch Standard-Farbe kommt vom Scalarvis-Server -> Client-Farben mitspeichern?
        // N TODO: Hintergrundfarbe der PlotArea aus Config holen?
        $this->PlotArea->SetBackgroundColor($this->ImageResource, 255, 255, 255);
        $this->PlotArea->SetMetrics($this->LeftMargin,
                                    $this->TopMargin,
                                    $this->ImageWidth - $this->LeftMargin - $this->RightMargin,
                                    $this->ImageHeight - $this->BottomMargin - $this->TopMargin);
        $this->PlotArea->Draw($this->ImageResource);

        // Anzahl-Tags-Relevanz
        if(isset($this->data[0][9])){
            $i = 0;
            foreach($this->data[0][9] as $tag_index => $tag_nr){
                $this->set_tagauswahl_index($tag_index, $tag_nr, $i);
                $i++;
            }
        }

        // Ribbonplot (scalarvis_ext_ribbon_plot), wenn SC.BlockInfo().AnzWerte ($this->data[0][4]) < SC.BlockInfo().anzval ($this->data[0][2])
        if(isset($this->data[0][9]) && isset($this->data[0][4]) && isset($this->data[0][2]) && isset($this->data[0][1])){
            // Wenn AnzWerte (zurckgeliefert) weniger als anzval (server) -> Ribbonplot
            if($this->is_ribbonplot){
                foreach($this->data[0][9] as $tag_index => $tag_nr){
                    $this->Plot[$tag_index] = new scalarvis_ext_ribbon_plot;
                    $this->Plot[$tag_index]->SetCodeBase($this->CodeBase);
                    $this->Plot[$tag_index]->SetNumberOfValues((int) floor($this->data[0][4]));
                    $this->Plot[$tag_index]->set_tag_info($this->tagauswahl[$tag_index]);
                    $transparency = 1.0 / pow($this->data[0][1], 0.5);
                    $this->Plot[$tag_index]->SetColorByTagInfo($this->ImageResource, $transparency);
                    $this->Plot[$tag_index]->SetMetrics($this->PlotArea->x1,
                                                        $this->PlotArea->y1,
                                                        $this->PlotArea->width,
                                                        $this->PlotArea->height);
                    $this->Plot[$tag_index]->SetRangeByTagInfo();
                    $this->Plot[$tag_index]->Draw($this->ImageResource);
                }
            }
            // Sonst Lineplot
            else{
                foreach($this->data[0][9] as $tag_index => $tag_nr){
                    $this->Plot[$tag_index] = new scalarvis_ext_line_plot;
                    $this->Plot[$tag_index]->SetCodeBase($this->CodeBase);
                    $this->Plot[$tag_index]->set_tag_info($this->tagauswahl[$tag_index]);
                    $this->Plot[$tag_index]->SetColorByTagInfo($this->ImageResource);
                    $this->Plot[$tag_index]->SetMetrics($this->PlotArea->x1,
                                                        $this->PlotArea->y1,
                                                        $this->PlotArea->width,
                                                        $this->PlotArea->height);
                    $this->Plot[$tag_index]->SetRangeByTagInfo();
                    $this->Plot[$tag_index]->Draw($this->ImageResource);
                }
            }
        }

        // Vertikal-Achsen ausgeben
        if(isset($this->data[0][9])){
            $RightPositioned = false;
            $LeftAxisPos = $this->PlotArea->x1;
            $RightAxisPos = $this->PlotArea->x2;
            foreach($this->data[0][9] as $tag_index => $tag_nr){
                $this->VertAxis[$tag_index] = new scalarvis_ext_vert_axis;
                $this->VertAxis[$tag_index]->SetCodeBase($this->CodeBase);
                if($this->is_shrinked){
                    $this->VertAxis[$tag_index]->SetMode(1);
                }
                $this->VertAxis[$tag_index]->set_tag_info($this->tagauswahl[$tag_index]);
                $this->VertAxis[$tag_index]->SetColorByTagInfo($this->ImageResource);

                if($RightPositioned){
                    $this->VertAxis[$tag_index]->SetMetrics($RightAxisPos,
                                                            $this->PlotArea->y1,
                                                            $this->PlotArea->height,
                                                            $RightPositioned);
                    $RightAxisPos += $this->AxisWidth;
                    $RightPositioned = false;
                }
                else{
                    $this->VertAxis[$tag_index]->SetMetrics($LeftAxisPos,
                                                            $this->PlotArea->y1,
                                                            $this->PlotArea->height,
                                                            $RightPositioned);
                    $LeftAxisPos -= $this->AxisWidth;
                    $RightPositioned = true;
                }

                $this->VertAxis[$tag_index]->SetRangeByTagInfo();
                $this->VertAxis[$tag_index]->Draw($this->ImageResource);

                // Bei SingleScale nur eine vertikale Achse zeichnen
                if($this->is_single_type()){
                    break;
                }
            }
        }

        // Zeitskala
        $this->HorAxis[0] = new scalarvis_ext_hor_axis;
        $this->HorAxis[0]->SetCodeBase($this->CodeBase);
        if($this->is_shrinked){
            $this->HorAxis[0]->SetMode(1);
        }
        $this->HorAxis[0]->SetColor($this->ImageResource, 0, 0, 0);
        $this->HorAxis[0]->SetMetrics($this->PlotArea->x1,
                                      $this->PlotArea->y2,
                                      $this->PlotArea->width);
        $this->HorAxis[0]->SetTimeframe($this->time_start,
                                        $this->time_end);
        $this->HorAxis[0]->Draw($this->ImageResource);

        return true;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion filter() - Hilfsfunktion fr AWT und AWT_STAT */
    private function filter($tok){
        $tok = doubleval($tok);
        if(!is_numeric($tok) || strstr($tok, '-1E+038')){
            $tok = 'undefined';
        }
        return $tok;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_awt_stat() - Liefert Archivdaten zur Mehrfachauslesung */
    private function get_awt_stat($index){
        $socket_buffer = sprintf('AWTSTAT,%d', $index);
        socket_write_all($this->socket, $socket_buffer);
        $socket_buffer = socket_read_all($this->socket, $this->MESSAGE_SIZE);
        if(stristr($socket_buffer, 'ERROR')){
            return 0;
        }
        else{
            $socket_buffer = substr($socket_buffer, 4);        // AWT, abschneiden
            $tok = strtok($socket_buffer, ',');
            $erg['index'] = $tok;       $tok = strtok(',');
            $erg['Datum'] = $tok;       $tok = strtok(',');
            $erg['Zeit'] = $tok;        $tok = strtok(',');
            $erg['anz_werte'] = $tok;   $tok = strtok(',');

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['wert_'.$j] = $this->filter($tok);
                $tok = strtok(',');
            }

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['min_'.$j] = $this->filter($tok);
                $tok = strtok(',');
            }

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['max_'.$j] = $this->filter($tok);
                $tok = strtok(',');
            }

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['sum_'.$j] = floatval($tok);
                $tok = strtok(',');
            }

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['anz_'.$j] = $this->filter($tok);
                $tok = strtok(',');
            }   // Gltige Datenpunkte im Zeitraum

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['avg_'.$j] = $this->filter($tok);
                $tok = strtok(',');
            }   // Mittelwerte

            for($j = 0; $j < $erg['anz_werte']; $j++){
                $erg['var_'.$j] = $this->filter($tok);
                $tok = strtok(',');
            }   // Varianz
        }

        return $erg;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_block_info() - Gibt die ntigen Informationen fr eine Blockdatenbertragung zurck (Archiv lesen) */
    /**
     * @return array(Auslesebeginn,anztag,anzval,StartOffset,AnzWerte,Intervall,Blocksize,Blockanz,Schrittweite,Tags[MAXAWTAGS])
     * @param sttzstellen integer(=die Menge an Sttzstellen, die zurckgeliefert werden)
     * wenn anzval == 0 dann ist die anfrage zulssig aber enthlt keine Werte
     */
    private function get_block_info($stuetzstellen){
        $retval = 0;

        if(is_numeric($stuetzstellen)){
            $socket_buffer = sprintf('GetBlockInfo,%d', $stuetzstellen);
            socket_write_all($this->socket, $socket_buffer);
            $socket_buffer = socket_read_all($this->socket, 304, true);
            if(!strncmp($socket_buffer, 'ERROR', 5)){
                die('Error on GetBlockInfo');
            }

            $temp = unpack('deins', substr($socket_buffer, 0, 8));  // Unpack liefert ein Array ohne Keys - deshalb einen Key 'eins' erstellen
            $erg[] = $this->ole_to_utc($temp['eins']);
            $temp = unpack('l4', substr($socket_buffer, 8, 16));    // Unpack liefert hier ein Array mit 4 Werten, also an erg anhngen...
            $erg = array_merge($erg, $temp);
            $temp = unpack('deins', substr($socket_buffer, 24, 8));
            $erg[] = $this->ole_to_utc($temp['eins']);
            $temp = unpack('l3', substr($socket_buffer, 32, 12));
            $erg = array_merge($erg, $temp);
            $temp = unpack('l'.$this->MAX_AW_TAGS, substr($socket_buffer, 44, 8 * $this->MAX_AW_TAGS));
            for($i = 1; $i <= $this->MAX_AW_TAGS; $i++){  // Tagliste durchlaufen
                if(!$temp[$i] == 0){  // und ungltige (=0) lschen
                    $temp1[] = $temp[$i];
                }
            }
            $erg[] = $temp1;
            //Berechnung des Auslesebeginns
            $erg[0] = $erg[0] + $erg[5] * $erg[3];    // Auslesebeginn = DatumVon + Intervall * Startoffset
            //Berechnung des Intervalls
            $erg[5] = $erg[5] * $erg[8];              // Intervall = Intervall * Schrittweite
            $retval = $erg;
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_block_type01() - Liest Blockdaten vom Typ 1 (Archiv lesen) */
    private function get_block_type01($blocknummer, $tagindex, $values_only = true){
        global $STANDARD_CONFIG;
        $retval = 0;
        $erg = array();
        $header_len = 12;

        if(isset($STANDARD_CONFIG['OLD_SOCKET_HANDLING'])
        && $STANDARD_CONFIG['OLD_SOCKET_HANDLING']){
            $old_socket_handling = true;
        }
        else{
            $old_socket_handling = false;
        }

        if(is_numeric($blocknummer) && is_numeric($tagindex)){
            if(!$old_socket_handling){
                $socket_buffer = sprintf('GetBlockType01,%d,%d', $blocknummer, $tagindex);
                socket_write_all($this->socket, $socket_buffer);
                $socket_buffer = socket_read_all($this->socket, $header_len, true);
                if(!strstr($socket_buffer, 'ERROR')){
                    //---------BlockHeader empfangen------------
                    $head = unpack('l3', $socket_buffer);
                    $anz_entries = (isset($head[1]) && is_numeric($head[1])?$head[1]:0);
                    $entry_size = (isset($head[3]) && is_numeric($head[3])?$head[3]:0);
                    $read_len = $anz_entries * $entry_size;
                    /*
                    echo "\n";
                    print_r($head);
                    echo "\n";
                    print_r($read_len);
                    if($blocknummer == 1) exit;
                    */
                    //---------BlockType01 empfangen------------
                    $socket_buffer = socket_read_all($this->socket, $read_len, true);
                    /*
                    echo "\n";
                    print_r(strlen($socket_buffer));
                    */
                    if(!strstr($socket_buffer, 'ERROR')){
                        $position = 0;
                        if($values_only){
                            $selection = 1;     // 0x1
                            for($i = 0; $i < $head[1]; $i++){
                                $erg = array_merge($erg,
                                                   unpack('d*',
                                                          substr($socket_buffer,
                                                                 $position,
                                                                 ((bool)($selection & 1)) * 8).
                                                          substr($socket_buffer,
                                                                 $position + 8,
                                                                 ((bool)($selection & 2)) * 8).
                                                          substr($socket_buffer,
                                                                 $position + 16,
                                                                 ((bool)($selection & 4)) * 8).
                                                          substr($socket_buffer,
                                                                 $position + 24,
                                                                 ((bool)($selection & 8)) * 8)));
                                $position += 32;
                            }
                        }
                        else{
                            $selection = 15;    // 0xF
                            for($i = 0; $i < $head[1]; $i++){
                                $temp = unpack('d*',
                                               substr($socket_buffer,
                                                      $position,
                                                      ((bool)($selection & 1)) * 8).
                                               substr($socket_buffer,
                                                      $position + 8,
                                                      ((bool)($selection & 2)) * 8).
                                               substr($socket_buffer,
                                                      $position + 16,
                                                      ((bool)($selection & 4)) * 8).
                                               substr($socket_buffer,
                                                      $position + 24,
                                                      ((bool)($selection & 8)) * 8));
                                $temp2 = array();
                                $temp2['Wert'] = $temp[1];
                                $temp2['Min'] = $temp[2];
                                $temp2['Max'] = $temp[3];
                                $temp2['Avg'] = $temp[4];
                                $erg[] = $temp2;

                                $position += 32;
                            }
                        }
                        $retval = $erg;
                    }
                }
            }
            // Alte Vorgehensweise
            else{
                $socket_buffer = sprintf('GetBlockType01,%d,%d', $blocknummer, $tagindex);
                socket_write_all($this->socket, $socket_buffer);
                $socket_buffer = socket_read($this->socket, $this->MESSAGE_SIZE);
                if(!strstr($socket_buffer, 'ERROR')){
                    //---------BlockHeader empfangen------------
                    $head = unpack('l3', $socket_buffer);
                    $position = 12;       // Lesepointer um die Lnge des Heads vorschieben
                    //---------BlockType01 empfangen------------
                    if($values_only){
                        $selection = 1;
                        for($i = 0; $i < $head[1]; $i++){
                            $erg = array_merge($erg,
                                               unpack('d*',
                                                      substr($socket_buffer,
                                                             $position,
                                                             ((bool)($selection & 1)) * 8).
                                                      substr($socket_buffer,
                                                             $position + 8,
                                                             ((bool)($selection & 2)) * 8).
                                                      substr($socket_buffer,
                                                             $position + 16,
                                                             ((bool)($selection & 4)) * 8).
                                                      substr($socket_buffer,
                                                             $position + 24,
                                                             ((bool)($selection & 8)) * 8)));
                            $position += 32;
                        }
                    }
                    else{
                        $selection = 15;
                        for($i = 0; $i < $head[1]; $i++){
                            $temp = unpack('d*',
                                           substr($socket_buffer,
                                                  $position,
                                                  ((bool)($selection & 1)) * 8).
                                           substr($socket_buffer,
                                                  $position + 8,
                                                  ((bool)($selection & 2)) * 8).
                                           substr($socket_buffer,
                                                  $position + 16,
                                                  ((bool)($selection & 4)) * 8).
                                           substr($socket_buffer,
                                                  $position + 24,
                                                  ((bool)($selection & 8)) * 8));
                            $temp2 = array();
                            $temp2['Wert'] = $temp[1];
                            $temp2['Min'] = $temp[2];
                            $temp2['Max'] = $temp[3];
                            $temp2['Avg'] = $temp[4];
                            $erg[] = $temp2;

                            $position += 32;
                        }
                    }
                    $retval = $erg;
                }
            }
        }

        return $retval;
    }
    /* }}} */
	
    /* {{{ PHP-Klassenfunktion get_block_info_js() - Gibt die ntigen Informationen fr eine Blockdatenbertragung zurck (Archiv lesen) */
    /**
     * @return array(Auslesebeginn,anztag,anzval,StartOffset,AnzWerte,Intervall,Blocksize,Blockanz,Schrittweite,Tags[MAXAWTAGS])
     * @param sttzstellen integer(=die Menge an Sttzstellen, die zurckgeliefert werden)
     * wenn anzval == 0 dann ist die anfrage zulssig aber enthlt keine Werte
     */
    private function get_block_info_js($stuetzstellen){
        $retval = 0;

        if(is_numeric($stuetzstellen)){
            $socket_buffer = sprintf('GetBlockInfo,%d', $stuetzstellen);
            socket_write_all($this->socket, $socket_buffer);
            $socket_buffer = socket_read_all($this->socket, 304, true);
            if(!strncmp($socket_buffer, 'ERROR', 5)){
                die('Error on GetBlockInfo');
            }

            $temp = unpack('deins', substr($socket_buffer, 0, 8));  // Unpack liefert ein Array ohne Keys - deshalb einen Key 'eins' erstellen
            $erg[] = $this->ole_to_utc($temp['eins']);//DatumVon
            $temp = unpack('l4', substr($socket_buffer, 8, 16));    // Unpack liefert hier ein Array mit 4 Werten, also an erg anhngen...
			$anztag = $temp[1];
            $erg = array_merge($erg, $temp);//AnzTAG, AnzVal, StartOffset, AnzWerte
            $temp = unpack('deins', substr($socket_buffer, 24, 8));
            $erg[] = $this->ole_to_utc($temp['eins']);//Intervall
            $temp = unpack('l3', substr($socket_buffer, 32, 12));
            $erg = array_merge($erg, $temp);//Blocksize, Blockanz, Schrittweite
            $temp = unpack('l'.$this->MAX_AW_TAGS, substr($socket_buffer, 44, 8 * $this->MAX_AW_TAGS));
            for($i = 1; $i <= $anztag; $i++){  // nur definierte Tagliste durchlaufen
                //if(!$temp[$i] == 0){  // und ungltige (=0) lschen
                    $temp1[$i] = $temp[$i];
                //}
            }
            $erg[] = $temp1;
            //Berechnung des Auslesebeginns
            $erg[0] = $erg[0] + $erg[5] * $erg[3];    // Auslesebeginn = DatumVon + Intervall * Startoffset
            //Berechnung des Intervalls
            $erg[5] = $erg[5] * $erg[8];              // Intervall = Intervall * Schrittweite
            $retval = $erg;
        }

        return $retval;
    }
    /* }}} */
	
    /* {{{ PHP-Klassenfunktion get_block_type01_js() - Liest Blockdaten vom Typ 1 (Archiv lesen) */
    private function get_block_type01_js($blocknummer, $tagindex, $values_only = true){
        $retval = 0;
        $erg = array();
        $header_len = 12;

		//$old_socket_handling = false;//wird nur noch sichere Socketverbindung verwendet
        if(is_numeric($blocknummer) && is_numeric($tagindex)){
			$socket_buffer = sprintf('GetBlockType01,%d,%d', $blocknummer, $tagindex);
			socket_write_all($this->socket, $socket_buffer);
			$socket_buffer = socket_read_all($this->socket, $header_len, true);
			if(!strstr($socket_buffer, 'ERROR')){
				//---------BlockHeader empfangen------------
				$head = unpack('l3', $socket_buffer);
				$anz_entries = (isset($head[1]) && is_numeric($head[1])?$head[1]:0);//Anzahl Werte
				$entry_size = (isset($head[3]) && is_numeric($head[3])?$head[3]:0);// Groesse eines Wertepaars
				$read_len = $anz_entries * $entry_size;//Laenge aller Werte eines TAGs
				/*
				echo "\n";
				print_r($head);
				echo "\n";
				var_dump($read_len);
//                    exit;
				*/
				//---------BlockType01 empfangen------------
				$socket_buffer = socket_read_all($this->socket, $read_len, true);
				if(!strstr($socket_buffer, 'ERROR')){
					$position = 0;
					$selection = 15;    // 0xF
					for($i = 0; $i < $anz_entries; $i++){//Anzahl Werte
						$temp = unpack('d*',
									   substr($socket_buffer,
											  $position,
											  ((bool)($selection & 1)) * 8).
									   substr($socket_buffer,
											  $position + 8,
											  ((bool)($selection & 2)) * 8).
									   substr($socket_buffer,
											  $position + 16,
											  ((bool)($selection & 4)) * 8).
									   substr($socket_buffer,
											  $position + 24,
											  ((bool)($selection & 8)) * 8));
						$temp2 = array();
						$temp2['Wert'] = $temp[1];
						$temp2['Min'] = $temp[2];
						$temp2['Max'] = $temp[3];
						$temp2['Avg'] = $temp[4];
						$erg[] = $temp2;

						$position += 32;
					}
					$retval = $erg;
				}
			}
        }
        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion get_diagram_data() - Daten fr Diagramm aufbereiten */
	 private function get_diagram_data(){
        $retval = false;
        $this->data = null;

        // Anzahl-Tags-Relevanz
        for($i = 0; $i < $this->svsc->number_of_tags; $i++){
            if(isset($this->svsc->tagauswahl[$i])
            && (!isset($this->svsc->tagauswahl[$i]['hide_tag']) || !$this->svsc->tagauswahl[$i]['hide_tag'])){
                $tags[] = $this->svsc->tagauswahl[$i]['nummer'];
            }
        }

        // N TODO: Vergleichsdiagramm vorsehen (mehrere Datenbereiche, mehrere Zeitrume) -> read_archiv() fr jeden Tag einzeln
        if(isset($tags)){
            if($this->is_single_type()){
                $this->Axes = 1;
            }
            else{
                $this->Axes = count($tags);
            }
            if($this->type == 'universal' || $this->type == 'single'){
                $this->data = $this->read_archiv(round_time($this->time_start, $this->sysinfo['intervall']),
                                                 round_time($this->time_end, $this->sysinfo['intervall']),
                                                 $tags,
                                                 500,
                                                 false);
            }
            if($this->type == 'detail' || $this->type == 'singled'){
                $this->data = $this->read_archiv(round_time($this->time_start, $this->sysinfo['stoer_intervall']),
                                                 round_time($this->time_end, $this->sysinfo['stoer_intervall']),
                                                 $tags,
                                                 500,
                                                 false,
                                                 'detail');
            }
            if(($this->type == 'trend') || ($this->type == 'singlet')){
                $this->data = $this->read_archiv(round_time($this->time_start, 60),
                                                 round_time($this->time_end, 60),
                                                 $tags,
                                                 500,
                                                 false,
                                                 'trend');
            }
        }
        else{
            $this->Axes = 0;
        }

        if(isset($this->data[0][0])){
            $this->time_start = $this->data[0][0];
            $this->time_end = $this->data[0][0] + ($this->data[0][4] - 1) * $this->data[0][5];
            $retval = true;
        }

        return $retval;
    }
    private function get_diagram_dataxx(){
        $retval = false;
        $this->data = null;

        // Anzahl-Tags-Relevanz
        for($i = 0; $i < $this->svsc->number_of_tags; $i++){
			$tags[$i] = 0;
            if(isset($this->svsc->tagauswahl[$i])
            && (!isset($this->svsc->tagauswahl[$i]['hide_tag']) || !$this->svsc->tagauswahl[$i]['hide_tag'])){
                $tags[$i] = $this->svsc->tagauswahl[$i]['nummer'];
            }
        }

        // N TODO: Vergleichsdiagramm vorsehen (mehrere Datenbereiche, mehrere Zeitrume) -> read_archiv() fr jeden Tag einzeln
        if(isset($tags)){
            if($this->is_single_type()){
                $this->Axes = 1;
            }
            else{
                $this->Axes = count($tags);
            }
            if($this->type == 'universal' || $this->type == 'single'){
                $this->data = $this->read_archiv_js(round_time($this->time_start, $this->sysinfo['intervall']),
                                                 round_time($this->time_end, $this->sysinfo['intervall']),
                                                 $tags,
                                                 500,
                                                 false);
            }
            if($this->type == 'detail' || $this->type == 'singled'){
                $this->data = $this->read_archiv_js(round_time($this->time_start, $this->sysinfo['stoer_intervall']),
                                                 round_time($this->time_end, $this->sysinfo['stoer_intervall']),
                                                 $tags,
                                                 500,
                                                 false,
                                                 'detail');
            }
            if(($this->type == 'trend') || ($this->type == 'singlet')){
                $this->data = $this->read_archiv_js(round_time($this->time_start, 60),
                                                 round_time($this->time_end, 60),
                                                 $tags,
                                                 1000,
                                                 false,
                                                 'trend');
            }
        }
        else{
            $this->Axes = 0;
        }

        if(isset($this->data[0][0])){
            $this->time_start = $this->data[0][0];
            $this->time_end = $this->data[0][0] + ($this->data[0][4] - 1) * $this->data[0][5];
            $retval = true;
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion ImageCreateErrorMessage() - Meldung als Grafik ausgeben */
    private function ImageCreateErrorMessage($text){
        if(!$this->ImageResource = @ImageCreate($this->ImageWidth, $this->ImageHeight)){
            return false;
        }
        $this->BackgroundColor = ImageColorAllocate($this->ImageResource, 200, 200, 200);
        $this->Color = ImageColorAllocate($this->ImageResource, 0, 0, 128);
        ImageString($this->ImageResource, 5, 10, 10, $text, $this->Color);

        return true;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion is_single_type() - Prft, ob Diagrammtyp einer der Single-Typen ist */
    private function is_single_type(){
        $retval = false;

        if(substr($this->type, 0, 6) == 'single'){
            $retval = true;
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion ole_to_utc() - Wandelt das Zeitformat OLE (Double) in UTC (Integer) um */
    private function ole_to_utc($ole){
        // 25569 = OLE-Zeitformat (Excel) 01.01.1970
        if($ole > 25569){
            $ole -= 25569;
        }
        return (int) round($ole * 86400);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion read_archiv() - Archiv lesen -> array(Blockinfo, Datenblock(Tag1), Datenblock(Tag2), ...) */
    private function read_archiv($from,
                                 $to,
                                 $tags,
                                 $amount = 500,
                                 $blockinfo_only = false,
                                 $data_type = 'univ'){
        global $STANDARD_CONFIG;

        // Fehlerabfrage ergnzen (< Maxtags)?
        if($data_type == 'detail'){
            $buf = $this->set_arcs($from, $to, $tags);
            if($buf){
                $anz = intval(substr($buf, 5, strlen($buf) - 24));  // String aufs Wesentliche reduzieren
            }
            else{
                return -1;
            }
        }
        else if($data_type == 'trend'){
            $buf = $this->set_trend($from, $to, $tags);
            if($buf){
                $anz = intval(substr($buf, 6, strlen($buf) - 24));  // String aufs Wesentliche reduzieren
            }
            else{
                return -1;
            }
        }
        else{   // universal
            $buf = $this->set_arct($from, $to, $tags);
            if($buf){
                $anz = intval(substr($buf, 5, strlen($buf) - 24));  // String aufs Wesentliche reduzieren
            }
            else{
                return -1;
            }
        }

        if($anz == 0){
            return 0;
        }
        else{
            $buf = $this->get_block_info($amount);      // Zeitintervall durch Systemintervall ergibt die Anzahl an Sttzstellen
            $erg[] = $buf;      // Blockinfo ist das erste Element des Ergebnisses, gefolgt von jeweils einem Element pro Tag
            //$buf=array(Auslesebeginn,anztag,anzval(server),StartOffset,AnzWerte(zurckgeliefert),Intervall,Blocksize,Blockanz,Schrittweite,Tags[MAXAWTAGS])
            if($blockinfo_only){
                return $erg;
            }
            // Wenn AnzWerte (zurckgeliefert) weniger als anzval (server) -> Daten fr Ribbonplot ermitteln (Wert, Min, Max, Avg)
            if(isset($erg[0][4]) && isset($erg[0][2]) && $erg[0][4] < $erg[0][2]
            && (!isset($STANDARD_CONFIG['NO_RIBBONPLOT']) || !$STANDARD_CONFIG['NO_RIBBONPLOT'])){
                $this->is_ribbonplot = true;
            }
            else{
                $this->is_ribbonplot = true; //false;
            }
            // Anzahl-Tags-Relevanz
            for($i = 0; $i < count($buf[9]); $i++){     // Fr jedes gltige Tag
                $temp = array();                        // $temp initialisieren
                if($this->is_ribbonplot){
                    for($j = 0; $j < $buf[7]; $j++){    // Fr jeden Datenblock
                        $temp = array_merge($temp, $this->get_block_type01($j, $i, false));    // Datenblcke zusammenfgen
                        //array_push($temp, $this->get_block_type01($j, $i, false));
                    }
                }
                else{
                    for($j = 0; $j < $buf[7]; $j++){    // Fr jeden Datenblock
                        $temp = array_merge($temp, $this->get_block_type01($j, $i, true));    // Datenblcke zusammenfgen
                    		//array_push($temp, $this->get_block_type01($j, $i, false));
                    }
                }
                $erg[] = $temp;
            }
            return $erg;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion read_archiv_js() - Archiv lesen -> array(Blockinfo, Datenblock(Tag1), Datenblock(Tag2), ...) */
    private function read_archiv_js($from,
                                 $to,
                                 $tags,
                                 $amount = 500,
                                 $blockinfo_only = false,
                                 $data_type = 'univ'){
        global $STANDARD_CONFIG;

        // Fehlerabfrage ergnzen (< Maxtags)?
        if($data_type == 'detail'){
            $buf = $this->set_arcs($from, $to, $tags);
            if($buf){
                $anz = intval(substr($buf, 5, strlen($buf) - 24));  // String aufs Wesentliche reduzieren
            }
            else{
                return -1;
            }
        }
        else if($data_type == 'trend'){
            $buf = $this->set_trend($from, $to, $tags);
            if($buf){
                $anz = intval(substr($buf, 6, strlen($buf) - 24));  // String aufs Wesentliche reduzieren
            }
            else{
                return -1;
            }
        }
        else{   // universal
            $buf = $this->set_arct($from, $to, $tags);
            if($buf){
                $anz = intval(substr($buf, 5, strlen($buf) - 24));  // String aufs Wesentliche reduzieren
            }
            else{
                return -1;
            }
        }

        if($anz == 0){
            return 0;
        }
        else{
            $buf = $this->get_block_info_js($amount);      // Zeitintervall durch Systemintervall ergibt die Anzahl an Sttzstellen
            $erg[] = $buf;      // Blockinfo ist das erste Element des Ergebnisses, gefolgt von jeweils einem Element pro Tag
            //$buf=array(Auslesebeginn,anztag,anzval(server),StartOffset,AnzWerte(zurckgeliefert),Intervall,Blocksize,Blockanz,Schrittweite,Tags[MAXAWTAGS])
            if($blockinfo_only){
                return $erg;
            }
			if(!isset($erg[0][4]) && !isset($erg[0][2])){
				return 0;
			}
            // Wenn AnzWerte (zurckgeliefert) weniger als anzval (server) -> Daten fr Ribbonplot ermitteln (Wert, Min, Max, Avg)
            $this->is_ribbonplot = true; //immer Ribbonplot lesen (Wert, Min, Max, Avg)
            // Anzahl-Tags-Relevanz
            for($i = 1; $i <= count($buf[9]); $i++){     // Fr jedes gltige Tag
                $temp = array();                        // $temp initialisieren
				if($buf[9][$i] > 0){
					if($this->is_ribbonplot){
						for($j = 0; $j < $buf[7]; $j++){    // Fr jeden Datenblock
							$temp = array_merge($temp, $this->get_block_type01_js($j, $i - 1, false));    // Datenblcke zusammenfgen
							//array_push($temp, $this->get_block_type01($j, $i, false));
						}
					}
				}
				//var_dump(count($temp));
                $erg[] = $temp;
            }
/*			
				echo "\n";
				print_r($erg);
				echo "\n";
                exit;
*/			
            return $erg;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion scalarvis_graph() - Daten ermitteln und Diagramm ausgeben (Hauptfunktion) */
    public function scalarvis_graph($svsc,
                                    $type,
                                    $returnImage = false,
                                    $imgSX = 0,
                                    $imgSY = 0,
                                    $CodeBase = ''){
        $retval = false;
        @ob_start();
        $this->svsc = $svsc;
        $this->time_start = $this->svsc->time_start;
        $this->time_end = $this->svsc->time_end;
        $this->type = $type;
        $this->returnImage = $returnImage;
        if($imgSX && $imgSY){
            if($imgSX < $this->ImageWidth || $imgSY < $this->ImageHeight){
                $this->is_shrinked = true;
                $this->LeftMargin = 10;
                $this->RightMargin = 10;
                $this->TopMargin = 20;
                $this->BottomMargin = 35;
            }
            $this->ImageWidth = $imgSX;
            $this->ImageHeight = $imgSY;
        }
        $this->CodeBase = $CodeBase;

        if(!$this->connect()){
            $ob = @ob_get_flush();
            @ob_end_clean();
            $this->ImageCreateErrorMessage($ob);
        }
        else{
            $this->data = null;
            if(!$this->get_diagram_data()){
                $ob = @ob_get_flush();
                @ob_end_clean();
                $this->ImageCreateErrorMessage($ob);
            }
            else{
                @ob_end_clean();
                if($this->Draw()){
                    $retval = true;
                }
                else{
                    $this->ImageCreateErrorMessage('Error drawing graph!');
                }
            }
        }

        if($this->returnImage){
            $retval = $this->ImageResource;
        }
        else{
            ImagePNG($this->ImageResource);
        }

        return $retval;
    }
    /* }}} */

	/* {{{ PHP-Klassenfunktion scalarvis_graph() - Daten ermitteln und Diagramm ausgeben (Hauptfunktion) */
    public function scalarvis_graph_GetData($svsc,
                                    $type,
                                    $returnImage = false,
                                    $imgSX = 0,
                                    $imgSY = 0,
                                    $CodeBase = ''){
        $retval = false;
		
		$ich = 0;
        @ob_start();
        $this->svsc = $svsc;
        $this->time_start = $this->svsc->time_start;
        $this->time_end = $this->svsc->time_end;

		$vorStartTrim=$this->time_start;
		$vorEndTrim=$this->time_end;

		$this->type = $type;
        $this->returnImage = $returnImage;
        if($imgSX && $imgSY){
            if($imgSX < $this->ImageWidth || $imgSY < $this->ImageHeight){
                $this->is_shrinked = true;
                $this->LeftMargin = 10;
                $this->RightMargin = 10;
                $this->TopMargin = 20;
                $this->BottomMargin = 35;
            }
            $this->ImageWidth = $imgSX;
            $this->ImageHeight = $imgSY;
        }
        $this->CodeBase = $CodeBase;
		//print_r_formatted("dsf");
		//print_r_formatted($svsc);
		//print_r_formatted("dsf");
		

        if(!$this->connect()){
            $ob = @ob_get_flush();
            @ob_end_clean();
            $this->ImageCreateErrorMessage($ob);
        }
        else{
			switch($type){
			case 'universal':
				$inter = $this->sysinfo['intervall'];
				break;
			case 'single':
				$inter = $this->sysinfo['intervall'];
				break;
			case 'detail':
				$inter = $this->sysinfo['stoer_intervall'];
				break;
			default:
				$inter = 60;
				break;
			}
			$svsc->time_start = trimTime($svsc->time_start, $inter);
			$svsc->time_end = trimTime($svsc->time_end, $inter);

			$this->data = null;
            if(!$this->get_diagram_dataxx()){
                $ob = @ob_get_flush();
                @ob_end_clean();
                $this->ImageCreateErrorMessage($ob);
            }
            else{
				$testArray['test'][0] = "YYY";
				$testArray['test'][1] = "<>";
				$testArray['test'][2] = $this->time_start;
				$testArray['test'][3] = $this->time_end;
				$testArray['test'][4] = $vorStartTrim;
				$testArray['test'][5] = $vorEndTrim;
				$testArray['test'][6] = $inter;
				$testArray['test'][7] = $type;
				
							$tempShip[0] = clone $svsc;
							$tempShip[1] = $this->data;
							$tempShip[2] = "X717";    //$svsc->tagauswahl[0]['einheit'];
							$tempShip[3] = $testArray;
							$tempShip[4] = NAN;
						
			                @ob_end_clean();
							 
						 	function checkNANSwapToken(&$item, $key)
							{
								if($item === '-1.#QNAN'){
											$item = -1.0E38;
								}
								else if ( is_numeric($item) ){
								 		if(is_nan($item)){
											$item = -1.0E38;
										}
										//$item = 123;
								}
								else if (is_string($item) ){
										$item = utf8_encode($item); 
								}
								else if(is_array($item)){
									//array_walk_recursive($item, 'checkNANSwapToken');
								}	
								else if(is_object($item)){
									array_walk_recursive($item, 'checkNANSwapToken');
								}	
							}		
							
							array_walk_recursive($tempShip, 'checkNANSwapToken');
											
							echo json_encode($tempShip);    
							
							$retval = true;
            }
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion server_command() - Setzen eines Server Commands */
    // Die Implementierung der Serverbefehle ist zur besseren Orientierung
    // mit Commands an das messvb1.ocx angelehnt und bereinstimmend
    private function server_command($cmd_nbr){
        $retval = false;

        switch($cmd_nbr){
        case 51:    // Server auf UTC setzen
            socket_write_all($this->socket, 'TIMEBASEUTC,');
            $socket_buffer = socket_read_all($this->socket, 512);
            if(stristr($socket_buffer, 'TIMEBASEUTC,***DONE***')){
                $retval = true;
            }
            break;
        case 52:    // Server auf Local time setzen
            socket_write_all($this->socket, 'TIMEBASELOCAL,');
            $socket_buffer = socket_read_all($this->socket, 512);
            if(stristr($socket_buffer, 'TIMEBASELOCAL,***DONE***')){
                $retval = true;
            }
            break;
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_arcs() - Detailauslesen der Archivdaten (Archiv lesen) */
    private function set_arcs($from, $to, $tags){
        if($to <= 0 || $from <= 0){
            return -1;
        }
        else{
            // Server auf UTC-Zeitrechnung setzen
            $this->server_command(51);

            $socket_buffer = sprintf('ARCS,%s,%s,%s,%s,%d,%s,',
                                     gmdate('d.m.y', $from),
                                     gmdate('H:i:s', $from),
                                     gmdate('d.m.y', $to),
                                     gmdate('H:i:s', $to),
                                     count($tags),
                                     $this->array_to_string($tags));
            socket_write_all($this->socket, $socket_buffer);
            return socket_read_all($this->socket, 512);
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_arct() - Universalauslesen der Archivdaten (Archiv lesen) */
    private function set_arct($from, $to, $tags){
        $retval = -1;

        if(is_numeric($from) && $from > 0
            && is_numeric($to) && $to > 0){
            // 51 Server auf UTC-Zeitrechnung setzen
            $this->server_command(51);

            $socket_buffer = sprintf('ARCT,%s,%s,%s,%s,%d,%s,',
                                     gmdate('d.m.y', $from),
                                     gmdate('H:i:s', $from),
                                     gmdate('d.m.y', $to),
                                     gmdate('H:i:s', $to),
                                     count($tags),
                                     $this->array_to_string($tags));
            socket_write_all($this->socket, $socket_buffer);
            $retval = socket_read_all($this->socket, 512);
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_tagauswahl() - Tag-Struktur bermitteln */
    private function set_tagauswahl_index($tag_index, $tag_nr, $i){
        $retval = false;

        foreach($this->svsc->tagauswahl as $tag_info){
            if($tag_info['nummer'] == $tag_nr){
                $this->tagauswahl[$tag_index] = $tag_info;

                // Autoscaling
                if(isset($this->AutoScale['min_'.$i])
                && is_numeric($this->AutoScale['min_'.$i])){
                    $this->tagauswahl[$tag_index]['dba'] = $this->AutoScale['min_'.$i];
                }
                if(isset($this->AutoScale['max_'.$i])
                && is_numeric($this->AutoScale['max_'.$i])){
                    $this->tagauswahl[$tag_index]['dbe'] = $this->AutoScale['max_'.$i];
                }

                // Referenz auf Daten-Array
                $data_index = $i + 1;   // Daten des ersten Tags beginnen bei Index 1
                if(isset($this->data[$data_index])){
                    $this->tagauswahl[$tag_index]['data_ref'] = &$this->data[$data_index];
                }

                $retval = true;
                break;
            }
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_trend() - Informationen fr Archivdaten-Lesen setzen -> entspricht der Detailkurve (Archiv lesen) */
    private function set_trend($from, $to, $tags){
        if($to <= 0 || $from <= 0){
            return -1;
        }
        else{
            // Server auf UTC-Zeitrechnung setzen
            $this->server_command(51);

            $socket_buffer = sprintf('TREND,%s,%s,%s,%s,%d,%s,',
                                     gmdate('d.m.y', $from),
                                     gmdate('H:i:s', $from),
                                     gmdate('d.m.y', $to),
                                     gmdate('H:i:s', $to),
                                     count($tags),
                                     $this->array_to_string($tags));
            socket_write_all($this->socket, $socket_buffer);
            return socket_read_all($this->socket, 512);
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetAutoScale() - AutoScale-Struktur-Array fllen */
    private function SetAutoScale(){
        $tagauswahl = array();

        for($i = 0; $i < $this->svsc->number_of_tags; $i++){
            if(isset($this->svsc->tagauswahl[$i])
            && (!isset($this->svsc->tagauswahl[$i]['hide_tag']) || !$this->svsc->tagauswahl[$i]['hide_tag'])){
                $tagauswahl[$i] = $this->svsc->tagauswahl[$i];
            }
        }

        ksort($tagauswahl);
        $keys = array_keys($tagauswahl);

        if(isset($this->data[0][2]) && is_numeric($this->data[0][2])
        && $this->data[0][2] > 0){    // (anzval)
            $this->AutoScale = $this->get_awt_stat(1);            // Autoscaling-Init
            $this->AutoScale = $this->get_awt_stat($this->data[0][2]);  // Autoscaling-Berechnung

            // Autoscaling verarbeiten
            $run = count($keys);
            for($i = 0, $j = 0; $i < $run; $i++){  // Autoscaling-Werte einsetzen
                // Negative Skalen erlaubt?
                if($this->AutoScale['min_'.$i] < 0 || $tagauswahl[$keys[$i]]['dba'] < 0){
                    if($this->svsc->USER_CONFIG['HIDE_NEG_AXIS'] && ($this->AutoScale['max_'.$i] > 0)){
                        $tagauswahl[$keys[$i]]['dba'] = 0;
                    }
                }
                // Usereingaben einsetzen oder AutoScale-Polster hinzufgen
                $percent_of_range = percent_of_range($this->AutoScale['min_'.$i], $this->AutoScale['max_'.$i], 10);
                if(isset($tagauswahl[$keys[$i]]['dba'])
                && (strlen(trim($tagauswahl[$keys[$i]]['dba'])))){
                    $this->AutoScale['min_'.$i] = $tagauswahl[$keys[$i]]['dba'];
                }
                else{
                    $this->AutoScale['min_'.$i] -= $percent_of_range;
                }
                if(isset($tagauswahl[$keys[$i]]['dbe'])
                && (strlen(trim($tagauswahl[$keys[$i]]['dbe'])))){
                    $this->AutoScale['max_'.$i] = $tagauswahl[$keys[$i]]['dbe'];
                }
                else{
                    $this->AutoScale['max_'.$i] += $percent_of_range;
                }

                // Bei Singlescale alle Achsen zusammenfassen
                if($this->is_single_type()){
                    if($j){
                        if($this->AutoScale['min_'.$i] < $temp_dba){
                            $temp_dba = $this->AutoScale['min_'.$i];
                        }
                        if($this->AutoScale['max_'.$i] > $temp_dbe){
                            $temp_dbe = $this->AutoScale['max_'.$i];
                        }
                    }
                    else{
                        $temp_dba = $this->AutoScale['min_'.$i];
                        $temp_dbe = $this->AutoScale['max_'.$i];
                    }
                    $j++;
                }
            }

            // Post-Processing: Errechnete Werte fr zusammengefasste Skalen einsetzen und zu lange Werte umformatieren
            for($i = 0; $i < $run; $i++){  // Autoscaling-Werte einsetzen
                if($this->is_single_type()){
                    $this->AutoScale['min_'.$i] = $temp_dba;
                    $this->AutoScale['max_'.$i] = $temp_dbe;
                }
                //$this->AutoScale['min_'.$i] = floor($this->AutoScale['min_'.$i]);
                //$this->AutoScale['max_'.$i] = ceil($this->AutoScale['max_'.$i]);
            }
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetGraphMargins() - Grafik-Rnder setzen */
    private function SetGraphMargins($left, $right, $top, $bottom){
        $this->LeftMargin = $left;
        $this->RightMargin = $right;
        $this->TopMargin = $top;
        $this->BottomMargin = $bottom;
    }
    /* }}} */
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_drawable - Basisklasse aller der in der Scalarvis-Extension darstellbaren Elemente */
class scalarvis_ext_drawable{
    protected $CodeBase = '';               // CodeBase, z.B. '../' (fr Fonts)
    protected $BackgroundColor = null;      // Hintergrundfarbe
    protected $Color = null;                // Vordergrundfarbe
    public    $height = 0;                  // Hhe des Elements
    public    $width = 0;                   // Breite des Elements

    /* {{{ PHP-Klassenfunktion SetBackgroundColor() - Hintergrundfarbe bestimmen */
    public function SetBackgroundColor(&$ImageResource, $Red, $Green, $Blue){
        $this->BackgroundColor = ImageColorAllocate($ImageResource, $Red, $Green, $Blue);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetCodeBase() - CodeBase bestimmen */
    public function SetCodeBase($CodeBase){
        $this->CodeBase = $CodeBase;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetColor() - Vordergrundfarbe bestimmen */
    public function SetColor(&$ImageResource, $Red, $Green, $Blue){
        $this->Color = ImageColorAllocate($ImageResource, $Red, $Green, $Blue);
    }
    /* }}} */
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_hor_axis - Horizontale Achse */
class scalarvis_ext_hor_axis extends scalarvis_ext_drawable{
    public    $x1 = 0;                        // X1-Position des Rechtecks
    public    $x2 = 0;                        // X2-Position des Rechtecks
    public    $y = 0;                         // Y-Position des Rechtecks
    private   $time_start = 0;                // Startzeit
    private   $time_end = 0;                  // Endezeit
    private   $time_int = 0;                  // Zeit-Intervall/Zeitspanne (Differenz zwischen Ende- und Startzeit)
    private   $SmallTickSize = 2;             // Kleine Tick-Gre
    private   $TickSize = 3;                  // Normale Tick-Gre
    private   $PtSize = 12;                   // Schriftgre
    private   $Mode = 0;                      // 0 = Gewinkelte Labels (Normaldarstellung), 1 = Horizontale Labels (Kleindarstellung)

    /* {{{ PHP-Klassenfunktion CalcTicks() - Ticks fr die Zeitachse berechnen */
    private function CalcTicks($MaxSections, &$Sections, &$TickStep, &$StartOffset){
        $MaxTicks = (int) floor($this->width / 70);     // Ursprnglich 80
        $StartInterval = 1;

        if($MaxTicks <= 1){
            $MaxTicks = 2;
        }

        if($this->time_int < 2592000){   // Zeitspanne kleiner als ein Monat
            $ScalingArray = array(5, 10, 20, 30, 40, 50, 60, 300, 900, 1800, 3600, 7200, 14400,
                                  21600, 28800, 36000, 43200, 50400, 57600, 64800, 72000, 79200, 86400);

            $tmp_last_index = count($ScalingArray) - 1;
            if($this->time_int < $MaxTicks * $ScalingArray[$tmp_last_index]){         // Ticksteps kleiner MaxTicks darstellbare Tage
                $TickStep = $StartInterval = $ScalingArray[$tmp_last_index];
                foreach($ScalingArray as $ScalingValue){
                    if($this->time_int / $ScalingValue < $MaxTicks){
                        $TickStep = $StartInterval = $ScalingValue;
                        break;
                    }
                }
            }
            else{                                   // Mehrere volle Tage als TickStep verwenden
                $TickStep = (int) floor(86400 * floor((double) $this->time_int / 86400 / $MaxTicks));
                $StartInterval = 86400;
            }
            $Sections = (int) floor($this->time_int / $TickStep);

            // Start-Offset berechnen
            $tmp_d = (double) $this->time_start / $StartInterval;
            if(floor($tmp_d) != $tmp_d){
                $tmp_d = floor($tmp_d) + 1;
            }
            $StartOffset = (int) ($tmp_d * $StartInterval) - $this->time_start;
        }
        else{
            $ScalingArray = array(432000, 864000, 1296000, 2592000, 5184000, 7776000, 15552000);
            //                    5 Tage  10 Tage 15 Tage  1 Monat  2 Monate 3 Monate 6 Monate

            $tmp_last_index = count($ScalingArray) - 1;
            if($this->time_int < $MaxTicks * $ScalingArray[$tmp_last_index]){
                $TickStep = $StartInterval = $ScalingArray[$tmp_last_index];
                foreach($ScalingArray as $ScalingValue){
                    if($this->time_int / $ScalingValue < $MaxTicks){
                        $TickStep = $StartInterval = $ScalingValue;
                        break;
                    }
                }
            }
            else{   // Full Year TickSteps
                $TickStep = (int) floor(2592000 * floor((double) $this->time_int / 2592000 / $MaxTicks));
                $StartInterval = 2592000;
            }
            $Sections = (int) floor($this->time_int / $TickStep);

            // Start-Offset berechnen
            $tm_mday = (int) localdate('j', $this->time_start);
            $tmp_e = (double) ($tm_mday * 86400) / $StartInterval;
            if(floor($tmp_e) != $tmp_e){
                $tmp_e = floor($tmp_e) + 1;
            }
            $StartOffset = (int) ($tmp_e * $StartInterval) - $tm_mday * 86400;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Draw() - Zeichnet die Achse */
    public function Draw(&$ImageResource){
        ImageLine($ImageResource,
                  $this->x1,
                  $this->y,
                  $this->x2,
                  $this->y,
                  $this->Color);
        $this->DrawTicks($ImageResource);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawTickLabel() - Tick-Beschriftung */
    private function DrawTickLabel(&$ImageResource, $x, $y, $val){
        switch($this->Mode){
        // Kleindarstellung
        case 1:
            ImageTTFText($ImageResource,
                         $this->PtSize - 1,
                         0,
                         $x - 22,
                         $y + 16,
                         $this->Color,
                         $this->CodeBase.'fonts/bluehigh.ttf',
                         localdate("d.m.y\nH:i:s", $val));
            break;
        // Normaldarstellung
        default:
            ImageTTFText($ImageResource,
                         $this->PtSize,
                         55,
                         $x - 20,
                         $y + 55,
                         $this->Color,
                         $this->CodeBase.'fonts/bluehigh.ttf',
                         localdate("d.m.y\n H:i:s", $val));
            break;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawTickLine() - Zeichnet einen Zeitskalen-Tick */
    private function DrawTickLine(&$ImageResource, $x, $Size){
        ImageLine($ImageResource,
                  $x,
                  $this->y - $Size,
                  $x,
                  $this->y + $Size,
                  $this->Color);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawTicks() - Zeichnet die Ticks der Zeitskala */
    private function DrawTicks(&$ImageResource){
        $Sections = 0;
        $TickStep = 0;
        $StartOffset = 0;

        $this->CalcTicks(10, $Sections, $TickStep, $StartOffset);

        $x = $this->x1;
        $y = $this->y;
        $val = $this->time_start + $StartOffset;

        if($TickStep / 2 <= $StartOffset){
            $this->DrawTickLine($ImageResource,
                                $this->TranslateX($this->Translate($val - $TickStep / 2)),
                                $this->SmallTickSize);
        }

        while($val <= $this->time_end){
            if($TickStep >= 2592000){               // Wenn grer als ein Monat
                $tmp_t_array['tm_sec'] = (int) localdate('s', $val);
                $tmp_t_array['tm_min'] = (int) localdate('i', $val);
                $tmp_t_array['tm_hour'] = (int) localdate('G', $val);
                $tmp_t_array['tm_mday'] = (int) localdate('j', $val);
                $tmp_t_array['tm_mon'] = (int) localdate('n', $val);
                $tmp_t_array['tm_year'] = (int) localdate('Y', $val);

                if($tmp_t_array['tm_mday'] < 4){
                    $tmp_t_array['tm_mday'] = 1;
                }
                else if($tmp_t_array['tm_mday'] > 11 && $tmp_t_array['tm_mday'] < 19){
                    $tmp_t_array['tm_mday'] = 15;
                }
                else if($tmp_t_array['tm_mday'] > 27){
                    $tmp_t_array['tm_mday'] = 1;
                    if($tmp_t_array['tm_mon'] == 12){
                        $tmp_t_array['tm_mon'] = 1;
                        $tmp_t_array['tm_year']++;
                    }
                    else{
                        $tmp_t_array['tm_mon']++;
                    }
                }

                $val = mktime($tmp_t_array['tm_hour'],
                              $tmp_t_array['tm_min'],
                              $tmp_t_array['tm_sec'],
                              $tmp_t_array['tm_mon'],
                              $tmp_t_array['tm_mday'],
                              $tmp_t_array['tm_year']);
            }

            $x = $this->TranslateX($this->Translate($val));
            $this->DrawTickLine($ImageResource, $x, $this->TickSize);
            $this->DrawTickLabel($ImageResource, $x, $y, $val);

            $perc = $this->Translate($val + $TickStep / 2);
            if($perc <= 1.0){
                $x = $this->TranslateX($perc);
                $this->DrawTickLine($ImageResource, $x, $this->SmallTickSize);
            }

            $val += $TickStep;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMetrics() - Mae bestimmen */
    public function SetMetrics($x, $y, $width){
        $this->width = $width;
        $this->x1 = $x;
        $this->x2 = $x + $this->width;
        $this->y = $y;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMode() - Modus bestimmen */
    public function SetMode($Mode){
        $this->Mode = $Mode;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetTimeframe() - Zeitraum bestimmen */
    public function SetTimeframe($time_start, $time_end){
        $this->time_start = $time_start;
        $this->time_end = $time_end;
        $this->time_int = $this->time_end - $this->time_start;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Translate() - Translate */
    private function Translate($val){
        return (double) ($this->time_int != 0?($val - $this->time_start) / $this->time_int:0);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion TranslateX() - TranslateX */
    private function TranslateX($fPercent){
        return (int) floor($this->x1 + $this->width * $fPercent);
    }
    /* }}} */
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_line_plot - Linien-Plot */
class scalarvis_ext_line_plot extends scalarvis_ext_drawable{
    public  $x1 = 0;                          // X1-Position des Zeichenbereichs
    public  $x2 = 0;                          // X2-Position des Zeichenbereichs
    public  $y1 = 0;                          // Y1-Position des Zeichenbereichs
    public  $y2 = 0;                          // Y2-Position des Zeichenbereichs

    private $dba = 0;                         // Datenbereichs-Start
    private $dbe = 0;                         // Datenbereichs-Ende
    private $Range = 0;                       // Datenbereichs-Differenz

    private $LinePosX = 0;                    // Aktuelle X-Position des Linien-Graphen
    private $LinePosY = 0;                    // Aktuelle Y-Position des Linien-Graphen

    private $tag_info = null;                 // Tag-Info-Array-Referenz

    /* {{{ PHP-Klassenfunktion Draw() - Zeichnet den Plot */
    public function Draw(&$ImageResource){
        $DrewPrevious = false;
        $LastNAN = true;
        $LastPercent = 0;
        $LastY = 0;

        $number_of_values = count($this->tag_info['data_ref']);
        if($number_of_values < 2){
            return;
        }

        $XPos = $this->x1;
        $TopY = $this->y1;
        $BottomY = $this->y2;
        $XStep = $this->width / ($number_of_values - 1);

        for($i = 0; $i < $number_of_values; $i++, $XPos += $XStep){
            $value = &$this->tag_info['data_ref'][$i];

            if($value == '-1.#QNAN' || !is_numeric($value) || is_nan($value)){
                $DrewPrevious = false;
                $LastNAN = true;
                continue;
            }

            $Percent = $this->Translate($value);
            $CurrY = $this->TranslateY($Percent);

            if($DrewPrevious){
//RS 20130522
                $x_intersect = 0.0;
                if($Percent >= 1.0){
                    $m = (double) ($LastY - $CurrY) / $XStep;
                    $t = (double) -$CurrY - $m * $XPos;
                    if($m != 0.0)
                    {
                    	$x_intersect = (double) (-$TopY - $t) / $m;
                    }
                    $this->DrawLineTo($ImageResource, (int) floor($x_intersect), $TopY);
                    $DrewPrevious = false;
                }
                else if($Percent <= 0.0){
                    $m = (double) ($LastY - $CurrY) / $XStep;
                    $t = (double) -$CurrY - $m * $XPos;
                    if($m != 0.0)
                    {
                    	$x_intersect = (double) (-$BottomY - $t) / $m;
                    }
                    $this->DrawLineTo($ImageResource, (int) floor($x_intersect), $BottomY);
                    $DrewPrevious = false;
                }
                else{
                    $this->DrawLineTo($ImageResource, $XPos, $CurrY);
                    $DrewPrevious = true;
                }
            }
            else{
                if($Percent > 0.0 && $Percent < 1.0){
                    if($LastNAN){
                        $this->SetPos((int) floor($XPos), $CurrY);
                        $LastNAN = false;
                        $DrewPrevious = true;
                    }
                    else{
//RS 20130522
						$x_intersect = 0.0;
                        if($LastPercent <= 0.0){
                            $m = (double) ($LastY - $CurrY) / $XStep;
                            $t = (double) -$CurrY - $m * $XPos;
							if($m != 0.0)
							{
								$x_intersect = (double) (-$BottomY - $t) / $m;
							}
                            $this->SetPos((int) floor($x_intersect), $BottomY);
                        }
                        else{
                            $m = (double) ($LastY - $CurrY) / $XStep;
                            $t = (double) -$CurrY - $m * $XPos;
							if($m != 0.0)
							{
	                            $x_intersect = (double) (-$TopY - $t) / $m;
							}
                            $this->SetPos((int) floor($x_intersect), $TopY);
                        }
                        $this->DrawLineTo($ImageResource, $XPos, $CurrY);
                        $DrewPrevious = true;
                    }
                }
                else{
                    if($Percent * $LastPercent < 0.0){
                        $m = (double) (-$LastY + $CurrY) / $XStep;
                        $t = (double) -$CurrY - $m * $XPos;
						$x_top_intersect = 0.0;
						$x_bottom_intersect = 0.0;
						if($m != 0.0)
						{
							$x_top_intersect = (double) (-$TopY - $t) / $m;
							$x_bottom_intersect = (double) (-$BottomY - $t) / $m;
						}
                        $this->SetPos((int) floor($x_top_intersect), $TopY);
                        $this->DrawLineTo($ImageResource, (int) floor($x_bottom_intersect), $BottomY);
                        $DrewPrevious = false;
                    }
                }
            }

            $LastPercent = $Percent;
            $LastY = $CurrY;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawLineTo() - Linie von der aktuellen Position des Linien-Graphen zur Zielposition zeichnen */
    private function DrawLineTo(&$ImageResource, $x, $y){
        ImageLine($ImageResource,
                  $this->LinePosX,
                  $this->LinePosY,
                  $x,
                  $y,
                  $this->Color);
        $this->LinePosX = $x;
        $this->LinePosY = $y;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_tag_info() - Tag-Info-Array-Referenz belegen */
    public function set_tag_info(&$tag_info_array){
        $this->tag_info = $tag_info_array;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetColorByTagInfo() - Vordergrundfarbe anhand Tag-Info bestimmen */
    public function SetColorByTagInfo(&$ImageResource){
        if(isset($this->tag_info['linecolor'])
        && is_numeric($this->tag_info['linecolor'])){
            $linecolor = (int) $this->tag_info['linecolor'];
        }
        else{
            $linecolor = 0;
        }
        $rgb_color_array = ForeColor2RGB($linecolor);
        $this->Color = ImageColorAllocate($ImageResource,
                                          $rgb_color_array[0],
                                          $rgb_color_array[1],
                                          $rgb_color_array[2]);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMetrics() - Mae bestimmen */
    public function SetMetrics($x, $y, $width, $height){
        $this->height = $height;
        $this->width = $width;
        $this->x1 = $x;
        $this->x2 = $x + $this->width;
        $this->y1 = $y;
        $this->y2 = $y + $this->height;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetPos() - Aktuelle Position des Linien-Graphen bestimmen */
    private function SetPos($x, $y){
        $this->LinePosX = $x;
        $this->LinePosY = $y;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetRangeByTagInfo() - Skalenbereich anhand Tag-Info bestimmen */
    public function SetRangeByTagInfo(){
        $this->dba = $this->tag_info['dba'];
        $this->dbe = $this->tag_info['dbe'];
        $this->Range = $this->tag_info['dbe'] - $this->tag_info['dba'];
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Translate() - Translate */
    private function Translate($val){
        return (double) ($this->Range != 0?($val - $this->dba) / $this->Range:0);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion TranslateX() - TranslateX */
    private function TranslateX($fPercent){
        return (int) floor($this->x1 + $this->width * $fPercent);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion TranslateY() - TranslateY */
    private function TranslateY($fPercent){
        return (int) floor($this->y2 - $this->height * $fPercent);
    }
    /* }}} */
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_plot_area - Zeichenbereich zur Darstellung des Graphen */
class scalarvis_ext_plot_area extends scalarvis_ext_drawable{
    public    $x1 = 0;                        // X1-Position des Rechtecks
    public    $x2 = 0;                        // X2-Position des Rechtecks
    public    $y1 = 0;                        // Y1-Position des Rechtecks
    public    $y2 = 0;                        // Y2-Position des Rechtecks

    /* {{{ PHP-Klassenfunktion Draw() - Zeichnet die Plot Area */
    public function Draw(&$ImageResource){
        ImageFilledRectangle($ImageResource,
                             $this->x1,
                             $this->y1,
                             $this->x2,
                             $this->y2,
                             $this->BackgroundColor);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMetrics() - Mae bestimmen */
    public function SetMetrics($x, $y, $width, $height){
        $this->height = $height;
        $this->width = $width;
        $this->x1 = $x;
        $this->x2 = $x + $this->width;
        $this->y1 = $y;
        $this->y2 = $y + $this->height;
    }
    /* }}} */
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_plot_buffer - Plot-Buffer-Struktur */
class scalarvis_ext_plot_buffer{
    public    $CurrY = 0;
    public    $LastY = 0;
    public    $CurrDrawX = 0;
    public    $CurrDrawY = 0;
    public    $CurrSetPosX = 0;
    public    $CurrSetPosY = 0;
    public    $CurrPercent = 0.0;
    public    $LastPercent = 0.0;
    public    $SetPos = false;
    public    $DrawPos = false;
    public    $DrewPrevious = false;
    public    $LastNAN = true;
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_point - Punkt */
class scalarvis_ext_point{
    public    $x = 0;
    public    $y = 0;
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_ribbon_plot - Ribbon-Plot */
class scalarvis_ext_ribbon_plot extends scalarvis_ext_drawable{
    public  $x1 = 0;                          // X1-Position des Zeichenbereichs
    public  $x2 = 0;                          // X2-Position des Zeichenbereichs
    public  $y1 = 0;                          // Y1-Position des Zeichenbereichs
    public  $y2 = 0;                          // Y2-Position des Zeichenbereichs

    private $transparency = 0;                // Transparenz
    private $ColorPolygon = null;             // Polygon-Farbe
    private $ColorMin = null;                 // Min-Line-Farbe
    private $ColorMax = null;                 // Max-Line-Farbe
    private $ColorAvg = null;                 // Avg-Line-Farbe

    private $dba = 0;                         // Datenbereichs-Start
    private $dbe = 0;                         // Datenbereichs-Ende
    private $Range = 0;                       // Datenbereichs-Differenz

    private $number_of_values = 0;            // Anzahl Werte
    private $XStep = 0;                       // X-Abstand zwischen den Werten
    private $XPos = 0;                        // Aktuelle X-Position
    private $TopY = 0;                        // Aktuelle obere Y-Position
    private $BottomY = 0;                     // Aktuelle untere Y-Position

    private $LinePosX = 0;                    // Aktuelle X-Position des Linien-Graphen
    private $LinePosY = 0;                    // Aktuelle Y-Position des Linien-Graphen

    private $tag_info = null;                 // Tag-Info-Array-Referenz

    /* {{{ PHP-Klassenfunktion CalcNextPoint() - Ermittelt die nchsten Koordinaten eines Plots */
    private function CalcNextPoint($value, &$buffer){
        // Letzte Werte merken
        $buffer->LastY = $buffer->CurrY;
        $buffer->LastPercent = $buffer->CurrPercent;

        // Alten Startpunkt setzen, um mehrere Plots zur selben Zeit ermglichen
        if($buffer->DrewPrevious && $buffer->DrawPos){
            $buffer->CurrSetPosX = $buffer->CurrDrawX;
            $buffer->CurrSetPosY = $buffer->CurrDrawY;
            $buffer->SetPos = true;
        }
        else{
            $buffer->SetPos = false;
        }

        $buffer->CurrPercent = $this->Translate($value);

        if($value == '-1.#QNAN' || !is_numeric($value) || is_nan($value)){
            $buffer->DrewPrevious = false;
            $buffer->LastNAN = true;
        }
        else{
            $buffer->CurrY = $this->TranslateY($buffer->CurrPercent);
            if($buffer->DrewPrevious){
                $buffer->SetPos = false;
                $buffer->DrawPos = true;
                if($buffer->CurrPercent >= 1.0){
                    $buffer->CurrDrawX = (int) floor($this->XPos - $this->XStep);
                    $buffer->CurrDrawY = $this->TopY;
                    $buffer->DrewPrevious = false;
                }
                else if($buffer->CurrPercent <= 0.0){
                    $buffer->CurrDrawX = (int) floor($this->XPos - $this->XStep);
                    $buffer->CurrDrawY = $this->BottomY;
                    $buffer->DrewPrevious = false;
                }
                else{
                    $buffer->CurrDrawX = (int) floor($this->XPos);
                    $buffer->CurrDrawY = $buffer->CurrY;
                    $buffer->DrewPrevious = true;
                }
            }
            else{
                $buffer->SetPos = true;
                if($buffer->CurrPercent >= 0.0 && $buffer->CurrPercent <= 1.0){
                    if($buffer->LastNAN){
                        $buffer->CurrSetPosX = (int) floor($this->XPos);
                        $buffer->CurrSetPosY = $buffer->CurrY;
                        $buffer->LastNAN = false;
                        $buffer->DrewPrevious = true;
                        $buffer->DrawPos = false;
                    }
                    else{
                        if($buffer->LastPercent <= 0.0){
                            $buffer->CurrSetPosX = (int) floor($this->XPos - $this->XStep);
                            $buffer->CurrSetPosY = $this->BottomY;
                        }
                        else{
                            $buffer->CurrSetPosX = (int) floor($this->XPos - $this->XStep);
                            $buffer->CurrSetPosY = $this->TopY;
                        }
                        $buffer->CurrDrawX = (int) floor($this->XPos);
                        $buffer->CurrDrawY = $buffer->CurrY;
                        $buffer->DrewPrevious = true;
                        $buffer->DrawPos = true;
                    }
                }
                else{
                    if(($buffer->CurrPercent * $buffer->LastPercent) <= 0.0){
                        $buffer->CurrSetPosX = (int) floor($this->XPos - $this->XStep);
                        $buffer->CurrSetPosY = $this->TopY;
                        $buffer->CurrDrawX = (int) floor($this->XPos - $this->XStep);
                        $buffer->CurrDrawY = $this->BottomY;
                        $buffer->DrewPrevious = false;
                    }
                }
            }
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Draw() - Zeichnet den Plot */
    public function Draw(&$ImageResource){
        $MinBuf = new scalarvis_ext_plot_buffer;
        $MaxBuf = new scalarvis_ext_plot_buffer;
        $AvgBuf = new scalarvis_ext_plot_buffer;

        $PolyPnt = 0;
        $PolyPts = array();
        for($i = 0; $i < 6; $i++){
            $PolyPts[$i] = new scalarvis_ext_point;
        }

        if($this->number_of_values < 2){
            return;
        }

        $this->XStep = $this->width / ($this->number_of_values - 1);
        $this->XPos = $this->TranslateX(0.0);
        $this->TopY = $this->y1;
        $this->BottomY = $this->y2;

        for($i = 0; $i < $this->number_of_values; $i++, $this->XPos += $this->XStep, $PolyPnt = 0){
            $value_struct = &$this->tag_info['data_ref'][$i];

            $this->CalcNextPoint($value_struct['Min'], $MinBuf);
            $this->CalcNextPoint($value_struct['Max'], $MaxBuf);
            $this->CalcNextPoint($value_struct['Avg'], $AvgBuf);

            // Zu fllende Polygone
            if(!$MinBuf->LastNAN && !$MaxBuf->LastNAN && $MinBuf->DrawPos && $MaxBuf->DrawPos){
                // MinPlot komplett innerhalb
                if($MinBuf->LastPercent <= 1.0 && $MinBuf->LastPercent >= 0.0
                && $MinBuf->CurrPercent <= 1.0 && $MinBuf->CurrPercent >= 0.0){
                    $PolyPts[$PolyPnt]->x = $MinBuf->CurrSetPosX;
                    $PolyPts[$PolyPnt++]->y = $MinBuf->CurrSetPosY;
                    $PolyPts[$PolyPnt]->x = $MinBuf->CurrDrawX;
                    $PolyPts[$PolyPnt++]->y = $MinBuf->CurrDrawY;
                }

                // MaxPlot komplett innerhalb
                if($MaxBuf->LastPercent <= 1.0 && $MaxBuf->LastPercent >= 0.0
                && $MaxBuf->CurrPercent <= 1.0 && $MaxBuf->CurrPercent >= 0.0){
                    $PolyPts[$PolyPnt]->x = $MaxBuf->CurrDrawX;
                    $PolyPts[$PolyPnt++]->y = $MaxBuf->CurrDrawY;
                    $PolyPts[$PolyPnt]->x = $MaxBuf->CurrSetPosX;
                    $PolyPts[$PolyPnt++]->y = $MaxBuf->CurrSetPosY;
                }

                // MinPlot komplett berhalb
                if($MinBuf->LastPercent > 1.0  && $MinBuf->CurrPercent > 1.0){
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos - $this->XStep);
                    $PolyPts[$PolyPnt++]->y = $this->TopY;
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos);
                    $PolyPts[$PolyPnt++]->y = $this->TopY;
                }
                // MinPlot komplett unterhalb
                else if($MinBuf->LastPercent < 0.0 && $MinBuf->CurrPercent < 0.0){
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos - $this->XStep);
                    $PolyPts[$PolyPnt++]->y = $this->BottomY;
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos);
                    $PolyPts[$PolyPnt++]->y = $this->BottomY;
                }

                // MaxPlot komplett berhalb
                if($MaxBuf->LastPercent > 1.0 && $MaxBuf->CurrPercent > 1.0 ){
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos);
                    $PolyPts[$PolyPnt++]->y = $this->TopY;
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos - $this->XStep);
                    $PolyPts[$PolyPnt++]->y = $this->TopY;
                }
                // MaxPlot komplett unterhalb
                else if($MaxBuf->LastPercent < 0.0 && $MaxBuf->CurrPercent < 0.0){
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos);
                    $PolyPts[$PolyPnt++]->y = $this->BottomY;
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos - $this->XStep);
                    $PolyPts[$PolyPnt++]->y = $this->BottomY;
                }

                // Letzter MinPlot innerhalb, gegenwrtiger auerhalb
                if($MinBuf->LastPercent <= 1.0 && $MinBuf->LastPercent >= 0.0
                && ($MinBuf->CurrPercent > 1.0 || $MinBuf->CurrPercent < 0.0)){
                    $PolyPts[$PolyPnt]->x = $MinBuf->CurrSetPosX;
                    $PolyPts[$PolyPnt++]->y = $MinBuf->CurrSetPosY;
                    $PolyPts[$PolyPnt]->x = $MinBuf->CurrDrawX;
                    $PolyPts[$PolyPnt++]->y = $MinBuf->CurrDrawY;
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos);
                    if($MinBuf->CurrPercent > 1.0){
                        $PolyPts[$PolyPnt++]->y = $this->TopY;
                    }
                    else{
                        $PolyPts[$PolyPnt++]->y = $this->BottomY;
                    }
                }

                // Letzter MaxPlot innerhalb, gegenwrtiger auerhalb
                if($MaxBuf->LastPercent <= 1.0 && $MaxBuf->LastPercent >= 0.0
                && ($MaxBuf->CurrPercent > 1.0 || $MaxBuf->CurrPercent < 0.0)){
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos);
                    if($MaxBuf->CurrPercent > 1.0){
                        $PolyPts[$PolyPnt++]->y = $this->TopY;
                    }
                    else{
                        $PolyPts[$PolyPnt++]->y = $this->BottomY;
                    }
                    $PolyPts[$PolyPnt]->x = $MaxBuf->CurrDrawX;
                    $PolyPts[$PolyPnt++]->y = $MaxBuf->CurrDrawY;
                    $PolyPts[$PolyPnt]->x = $MaxBuf->CurrSetPosX;
                    $PolyPts[$PolyPnt++]->y = $MaxBuf->CurrSetPosY;
                }

                // Letzter MinPlot auerhalb, gegenwrtiger innerhalb
                if(($MinBuf->LastPercent > 1.0 || $MinBuf->LastPercent < 0.0)
                && $MinBuf->CurrPercent <= 1.0 && $MinBuf->CurrPercent >= 0.0){
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos - $this->XStep);
                    if($MinBuf->LastPercent > 1.0){
                        $PolyPts[$PolyPnt++]->y = $this->TopY;
                    }
                    else{
                        $PolyPts[$PolyPnt++]->y = $this->BottomY;
                    }
                    $PolyPts[$PolyPnt]->x = $MinBuf->CurrSetPosX;
                    $PolyPts[$PolyPnt++]->y = $MinBuf->CurrSetPosY;
                    $PolyPts[$PolyPnt]->x = $MinBuf->CurrDrawX;
                    $PolyPts[$PolyPnt++]->y = $MinBuf->CurrDrawY;
                }

                // Letzter MaxPlot auerhalb, gegenwrtiger innerhalb
                if(($MaxBuf->LastPercent > 1.0 || $MaxBuf->LastPercent < 0.0)
                && $MaxBuf->CurrPercent <= 1.0 && $MaxBuf->CurrPercent >= 0.0){
                    $PolyPts[$PolyPnt]->x = $MaxBuf->CurrDrawX;
                    $PolyPts[$PolyPnt++]->y = $MaxBuf->CurrDrawY;
                    $PolyPts[$PolyPnt]->x = $MaxBuf->CurrSetPosX;
                    $PolyPts[$PolyPnt++]->y = $MaxBuf->CurrSetPosY;
                    $PolyPts[$PolyPnt]->x = (int) floor($this->XPos - $this->XStep);
                    if($MaxBuf->LastPercent > 1.0){
                        $PolyPts[$PolyPnt++]->y = $this->TopY;
                    }
                    else{
                        $PolyPts[$PolyPnt++]->y = $this->BottomY;
                    }
                }
                $PointArray = $this->PolyPts2PointArray($PolyPts, $PolyPnt);

                ImageFilledPolygon($ImageResource, $PointArray, $PolyPnt, $this->ColorPolygon);
            }

            // Min-Line
            $this->SetPos($MinBuf->CurrSetPosX, $MinBuf->CurrSetPosY);
            if($MinBuf->DrawPos){
                $this->DrawLineTo($ImageResource,
                                  $MinBuf->CurrDrawX,
                                  $MinBuf->CurrDrawY,
                                  $this->ColorMin);
            }

            // Max-Line
            $this->SetPos($MaxBuf->CurrSetPosX, $MaxBuf->CurrSetPosY);
            if($MaxBuf->DrawPos){
                $this->DrawLineTo($ImageResource,
                                  $MaxBuf->CurrDrawX,
                                  $MaxBuf->CurrDrawY,
                                  $this->ColorMax);
            }

            // Avg-Line
            $this->SetPos($AvgBuf->CurrSetPosX, $AvgBuf->CurrSetPosY);
            if($AvgBuf->DrawPos){
                $this->DrawLineTo($ImageResource,
                                  $AvgBuf->CurrDrawX,
                                  $AvgBuf->CurrDrawY,
                                  $this->ColorAvg);
            }
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawLineTo() - Linie von der aktuellen Position des Linien-Graphen zur Zielposition zeichnen */
    private function DrawLineTo(&$ImageResource, $x, $y, &$color = null){
        if($color == null){
            $color = $this->Color;
        }

        ImageLine($ImageResource,
                  $this->LinePosX,
                  $this->LinePosY,
                  $x,
                  $y,
                  $color);

        $this->LinePosX = $x;
        $this->LinePosY = $y;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion PolyPts2PointArray() - Konvertiert Polygonpunkt-Struktur fr ImageFilledPolygon() */
    private function PolyPts2PointArray($PolyPts, $PolyPnt){
        $retval = array();

        for($i = 0; $i < $PolyPnt; $i++){
            $retval[] = $PolyPts[$i]->x;
            $retval[] = $PolyPts[$i]->y;
        }

        return $retval;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_tag_info() - Tag-Info-Array-Referenz belegen */
    public function set_tag_info(&$tag_info_array){
        $this->tag_info = $tag_info_array;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetColorByTagInfo() - Vordergrundfarbe anhand Tag-Info bestimmen */
    public function SetColorByTagInfo(&$ImageResource, $transparency = 0){
        if(isset($this->tag_info['linecolor'])
        && is_numeric($this->tag_info['linecolor'])){
            $linecolor = (int) $this->tag_info['linecolor'];
        }
        else{
            $linecolor = 0;
        }
        $rgb_color_array = ForeColor2RGB($linecolor);

        $this->Color = ImageColorAllocate($ImageResource,
                                          $rgb_color_array[0],
                                          $rgb_color_array[1],
                                          $rgb_color_array[2]);

        $this->transparency = $transparency;

        if($this->transparency){
            $this->ColorPolygon = ImageColorAllocateAlpha($ImageResource,
                                                          $rgb_color_array[0],
                                                          $rgb_color_array[1],
                                                          $rgb_color_array[2],
                                                          (int) floor(127 - 127 * $transparency * 0.1));

            $this->ColorMin = ImageColorAllocateAlpha($ImageResource,
                                                      $rgb_color_array[0],
                                                      $rgb_color_array[1],
                                                      $rgb_color_array[2],
                                                      (int) floor(127 - 127 * $transparency * 0.2));

            $this->ColorMax = ImageColorAllocateAlpha($ImageResource,
                                                      $rgb_color_array[0],
                                                      $rgb_color_array[1],
                                                      $rgb_color_array[2],
                                                      (int) floor(127 - 127 * $transparency * 0.2));

            $this->ColorAvg = ImageColorAllocateAlpha($ImageResource,
                                                      $rgb_color_array[0],
                                                      $rgb_color_array[1],
                                                      $rgb_color_array[2],
                                                      (int) floor(127 - 127 * $transparency));
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMetrics() - Mae bestimmen */
    public function SetMetrics($x, $y, $width, $height){
        $this->height = $height;
        $this->width = $width;
        $this->x1 = $x;
        $this->x2 = $x + $this->width;
        $this->y1 = $y;
        $this->y2 = $y + $this->height;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetNumberOfValues() - Anzahl Werte bestimmen */
    public function SetNumberOfValues($number_of_values){
        $this->number_of_values = $number_of_values;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetPos() - Aktuelle Position des Linien-Graphen bestimmen */
    private function SetPos($x, $y){
        $this->LinePosX = $x;
        $this->LinePosY = $y;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetRangeByTagInfo() - Skalenbereich anhand Tag-Info bestimmen */
    public function SetRangeByTagInfo(){
        $this->dba = $this->tag_info['dba'];
        $this->dbe = $this->tag_info['dbe'];
        $this->Range = $this->tag_info['dbe'] - $this->tag_info['dba'];
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Translate() - Translate */
    private function Translate($val){
        return (double) ($this->Range != 0?($val - $this->dba) / $this->Range:0);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion TranslateX() - TranslateX */
    private function TranslateX($fPercent){
        return (int) floor($this->x1 + $this->width * $fPercent);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion TranslateY() - TranslateY */
    private function TranslateY($fPercent){
        return (int) floor($this->y2 - $this->height * $fPercent);
    }
    /* }}} */
}
/* }}} */

/* {{{ PHP-Klasse scalarvis_ext_vert_axis - Vertikale Achse */
class scalarvis_ext_vert_axis extends scalarvis_ext_drawable{
    public    $x = 0;                         // X-Position des Rechtecks
    public    $y1 = 0;                        // Y1-Position des Rechtecks
    public    $y2 = 0;                        // Y2-Position des Rechtecks

    private   $SmallTickSize = 2;             // Kleine Tick-Gre
    private   $TickSize = 3;                  // Normale Tick-Gre
    private   $PtSize = 11;                   // Schriftgre
    private   $RightPositioned = false;       // Rechts positioniert
    private   $Mode = 0;                      // 0 = Normaldarstellung, 1 = Kleindarstellung
    private   $tag_info = null;               // Tag-Info-Array-Referenz

    private   $dba = 0;                       // Datenbereichs-Start
    private   $dbe = 0;                       // Datenbereichs-Ende
    private   $Range = 0;                     // Datenbereichs-Differenz

    /* {{{ PHP-Klassenfunktion CalcTicks() - Ticks fr die Vertikalachse berechnen */
    private function CalcTicks($MaxSections, &$TickStep){
        $Sections = $MaxSections + 1;
        for($exp = -4; $Sections > $MaxSections; $exp++){
            $div = array(1, 2, 5);
            for($i = 0; $i < 3 && $Sections > $MaxSections; $i++){
                //$TickStep = (int) floor(pow(10.0, (double) $exp) * $div[$i]);
                $TickStep = pow(10.0, (double) $exp) * $div[$i];
                /*
$tmp_array = array('MaxSections' => $MaxSections,
                   'Sections' => $Sections,
                   'TickStep' => $TickStep,
                   'exp' => $exp,
                   'div' => $div,
                   'test' => pow(10.0, (double) $exp),
                   'Range' => $this->Range);
print_r_formatted($tmp_array); exit;
                 */
                if($TickStep == 0){
                    continue;
                }
                $Sections = $this->Range / $TickStep;
            }
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion Draw() - Zeichnet die Achse */
    public function Draw(&$ImageResource){
        ImageLine($ImageResource,
                  $this->x,
                  $this->y1,
                  $this->x,
                  $this->y2,
                  $this->Color);
        if($this->dbe > $this->dba){
            $this->DrawTicks($ImageResource);
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawTickLine() - Zeichnet einen Vertikalachsen-Tick */
    private function DrawTickLine(&$ImageResource, $y, $Size){
        ImageLine($ImageResource,
                  $this->x - $Size,
                  $y,
                  $this->x + $Size,
                  $y,
                  $this->Color);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion DrawTicks() - Zeichnet die Ticks der Vertikalachse */
    private function DrawTicks(&$ImageResource){
        $TickStep = 0;

        switch($this->Mode){
        // Kleindarstellung
        case 1:
            $this->CalcTicks(6, $TickStep);
            break;
        // Normaldarstellung
        default:
            $this->CalcTicks(10, $TickStep);
            break;
        }

        $Sections = $this->Range / $TickStep;
        $PixelsPerTick = (double) ($this->y2 - $this->y1) / $Sections;
        $y = $this->y2;
        $x = $this->x;
        $val = $this->dba;
        $TickOffs = $this->RightPositioned?15:-15;

        switch($this->Mode){
        // Kleindarstellung
        case 1:
            break;
        // Normaldarstellung
        default:
            ImageTTFText($ImageResource,
                         $this->PtSize,
                         55,
                         $this->RightPositioned?$x + 20:$x - 20,
                         $y + 50,
                         $this->Color,
                         $this->CodeBase.'fonts/bluehigh.ttf',
                         (!empty($this->tag_info['einheit'])?$this->tag_info['einheit']:''));
            break;
        }

        $StartOffset = 0;
        if($this->dbe * $this->dba < 0 && $TickStep != 0){
            $StartOffset = fmod(abs($this->dba), $TickStep);
            $val += $StartOffset;
            $y -= $PixelsPerTick * $StartOffset / $TickStep;
        }

        $decimal_places = optimum_decimal_places(($this->dba + $this->dbe) / 2, -1);

        while($y >= $this->y1){
            $this->DrawTickLine($ImageResource, $y, $this->TickSize);

            switch($this->Mode){
            // Kleindarstellung
            case 1:
                ImageTTFText($ImageResource,
                             $this->PtSize - 1,
                             75,
                             $x + $TickOffs,
                             $y + 10,
                             $this->Color,
							 $this->CodeBase.'fonts/bluehigh.ttf',
                             number_format($val, $decimal_places, '.', ''));
                break;
            // Normaldarstellung
            default:
                ImageTTFText($ImageResource,
                             $this->PtSize,
                             75,
                             $x + $TickOffs,
                             $y + 10,
                             $this->Color,
                             $this->CodeBase.'fonts/bluehigh.ttf',
                             number_format($val, $decimal_places, '.', ''));
                break;
            }

            if($y - $PixelsPerTick / 2 >= $this->y1){
                $this->DrawTickLine($ImageResource, (int) floor($y - $PixelsPerTick / 2), $this->SmallTickSize);
            }

            $val += $TickStep;
            $y -= $PixelsPerTick;
        }
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion set_tag_info() - Tag-Info-Array-Referenz belegen */
    public function set_tag_info(&$tag_info_array){
        $this->tag_info = $tag_info_array;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetColorByTagInfo() - Vordergrundfarbe anhand Tag-Info bestimmen */
    public function SetColorByTagInfo(&$ImageResource){
        if(isset($this->tag_info['linecolor'])
        && is_numeric($this->tag_info['linecolor'])){
            $linecolor = (int) $this->tag_info['linecolor'];
        }
        else{
            $linecolor = 0;
        }
        $rgb_color_array = ForeColor2RGB($linecolor);
        $this->Color = ImageColorAllocate($ImageResource,
                                          $rgb_color_array[0],
                                          $rgb_color_array[1],
                                          $rgb_color_array[2]);
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMetrics() - Mae bestimmen */
    public function SetMetrics($x, $y, $height, $RightPositioned){
        $this->height = $height;
        $this->x = $x;
        $this->y1 = $y;
        $this->y2 = $y + $this->height;
        $this->RightPositioned = $RightPositioned;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetMode() - Modus bestimmen */
    public function SetMode($Mode){
        $this->Mode = $Mode;
    }
    /* }}} */

    /* {{{ PHP-Klassenfunktion SetRangeByTagInfo() - Skalenbereich anhand Tag-Info bestimmen */
    public function SetRangeByTagInfo(){
        $this->dba = $this->tag_info['dba'];
        $this->dbe = $this->tag_info['dbe'];
        $this->Range = $this->tag_info['dbe'] - $this->tag_info['dba'];
    }
    /* }}} */
}
/* }}} */
?>
